@@ -1,11 +1,11 @@
-# Copyright 2021 Q-CTRL. All rights reserved.
+# Copyright 2022 Q-CTRL. All rights reserved.
 #
 # Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
 # copying or use of this file, via any medium, is strictly prohibited.
 # Proprietary and confidential. You may not use this file except in compliance
 # with the License. You may obtain a copy of the License at
 #
-#      https://q-ctrl.com/terms
+#    https://q-ctrl.com/terms
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS. See the
@@ -20,6 +20,7 @@ import forge
 
 from qctrlcommons.exceptions import QctrlArgumentsValueError
 from qctrlcommons.node.base import Node
+from qctrlcommons.node.documentation import Category
 from qctrlcommons.node.node_data import Pwc
 from qctrlcommons.node.wrapper import NamedNodeData
 
@@ -28,10 +29,7 @@ class GetAttributeNode(Node):
     """
     Gets an attribute from a node value.
 
-    Only certain combinations of `value` and `attr` are supported:
-
-    `Pwc.values`
-        The tensor of values of a `Pwc`.
+    Typically you would use the syntax ``value.attr`` instead of using this function directly.
 
     Parameters
     ----------
@@ -45,15 +43,20 @@ class GetAttributeNode(Node):
     Returns
     -------
     Any
-        The item or items obtained from `value.attr`, or to be more precise,
+        The item or items obtained from ``value.attr``, or to be more precise,
         ``getattr(value, attr)``.
+
+    Notes
+    -----
+    Only certain combinations of `value` and `attr` are supported:
+
+    ``Pwc.values``
+        The tensor of values of a `Pwc`.
     """
 
     name = "getattr"
-    args = [
-        forge.arg("value"),
-        forge.arg("attr", type=str),
-    ]
+    args = [forge.arg("value"), forge.arg("attr", type=str)]
+    categories = [Category.OTHER_OPERATIONS]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -105,6 +108,7 @@ class GetItemNode(Node):
         forge.arg("value"),
         forge.arg("key", type=Union[int, slice, List[Union[int, slice]]]),
     ]
+    categories = [Category.OTHER_OPERATIONS]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
