@@ -1,11 +1,11 @@
-# Copyright 2021 Q-CTRL. All rights reserved.
+# Copyright 2022 Q-CTRL. All rights reserved.
 #
 # Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
 # copying or use of this file, via any medium, is strictly prohibited.
 # Proprietary and confidential. You may not use this file except in compliance
 # with the License. You may obtain a copy of the License at
 #
-#      https://q-ctrl.com/terms
+#    https://q-ctrl.com/terms
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS. See the
@@ -17,21 +17,24 @@ from typing import Optional
 import forge
 
 from qctrlcommons.exceptions import QctrlException
+from qctrlcommons.node.documentation import Category
 from qctrlcommons.preconditions import check_argument
 
 
 def deprecated_node(
-    updated_node_name: Optional[str] = None,
-    extra_info: Optional[str] = None,
+    updated_node_name: Optional[str] = None, extra_info: Optional[str] = None
 ):
     """
     Creates a decorator to mark a node as deprecated. If the deprecation is only a renaming, the
     deprecated node class can be made a subclass of the new node class and left otherwise empty.
 
+    When you mark a node as deprecated, remember to remove the existing "See Also" references to it
+    or redirect them to the node with the updated name.
+
     Parameters
     ----------
     updated_node_name : str, optional
-        The name of the node to use instead of the the deprecated node. Defaults to None, in which
+        The name of the node to use instead of the deprecated node. Defaults to None, in which
         case no alternative is provided in the docstring or when calling the node.
     extra_info : str, optional
         Additional information to provide in the node's docstring and warning when it is called.
@@ -63,6 +66,9 @@ def deprecated_node(
         # Mark node as deprecated so it can be skipped in some tests.
         node.is_deprecated = True
 
+        # Ensure the node is in the deprecated documentation category.
+        node.categories = [Category.DEPRECATED_OPERATIONS]
+
         # Update create_node_data to throw a warning if called.
         original_create_node_data = node.create_node_data
 
@@ -82,11 +88,11 @@ def deprecated_node(
     return decorator
 
 
-def deprecated_parameter(deprecated_name: str, new_name: str):
+def rename_parameter(old_name: str, new_name: str):
     """
-    Creates a decorator to mark a parameter as deprecated. Use this to mark an argument
-    as deprecated on a node written only in terms of the "new" parameter.
-    You can deprecate multiple parameters by sequentially applying the decorator for each one.
+    Creates a decorator to mark a parameter as deprecated due to it being renamed. Use this to
+    mark an argument as deprecated on a node written only in terms of the new parameter name.
+    You can rename multiple parameters by sequentially applying the decorator for each one.
 
     The following cases are not supported by this decorator:
         - Parameter is required and occurs before other required parameters.
@@ -94,25 +100,30 @@ def deprecated_parameter(deprecated_name: str, new_name: str):
 
     Parameters
     ----------
-    deprecated_name : str
-        The deprecated parameter.
+    old_name : str
+        The original name of the deprecated parameter.
     new_name : str
         The parameter name to replace the deprecated one.
 
     Returns
     -------
     callable
-        A callable (decorator) that accepts a node class, adds the deprecated parameter to its
+        A callable (decorator) that accepts a node class, adds the old parameter name to its
         parameter list in the docstring and to the forge arguments, and wraps its `create_node_data`
-        to pass the updated parameter if the deprecated parameter is passed (throwing a warning).
+        to pass the updated parameter if the old parameter name is passed (throwing a warning).
+
+    Raises
+    ------
+    QctrlException
+        if the docstring of node does not include a Returns section.
 
     Example
     -------
-    @deprecated_parameter(  # deprecated 2019/08/24
-        deprecated_name="pulses_count", new_name="pulse_count"
+    @rename_parameter(  # deprecated 2019/08/24
+        old_name="number_of_pulses", new_name="pulse_count"
     )
     class FunctionNode(Node):
-        # node definition in terms of pulse_count
+        # node definition in terms of only pulse_count
     """
 
     def decorator(node):
@@ -128,16 +139,16 @@ def deprecated_parameter(deprecated_name: str, new_name: str):
         docstring_end = returns_str + docstring_end
 
         # Add deprecated parameter to docstring.
-        deprecated_str = (
-            f"\n    {deprecated_name} : deprecated"
+        old_parameter_str = (
+            f"\n    {old_name} : deprecated"
             "\n        This parameter will be removed in the future,"
             f"\n        please use `{new_name}` instead."
         )
-        docstring = docstring + deprecated_str
+        docstring = docstring + old_parameter_str
 
         node.__doc__ = docstring + docstring_end
 
-        # Check if the updated parameter is required (i.e. has an empty default).
+        # Check if the renamed parameter is required (i.e. has an empty default).
         # If so, add a None default.
         node_args = []
         for forge_arg in node.args:
@@ -149,24 +160,24 @@ def deprecated_parameter(deprecated_name: str, new_name: str):
             node_args.append(forge_arg)
         node.args = node_args
 
-        # Add deprecated parameter as a keyword parameter.
+        # Add deprecated parameter name as a keyword parameter.
         node_kwargs = node.kwargs.copy()
-        node_kwargs[deprecated_name] = forge.kwarg(
-            deprecated_name, type=Optional[arg_type], default=None
+        node_kwargs[old_name] = forge.kwarg(
+            old_name, type=Optional[arg_type], default=None
         )
         node.kwargs = node_kwargs
 
-        # Update create_node_data to throw a warning if called with the deprecated parameter.
+        # Update create_node_data to throw a warning if called with the deprecated parameter name.
         original_create_node_data = node.create_node_data
 
         def new_create_node_data(*args, **kwargs):
-            deprecated_value = kwargs[deprecated_name]
-            if deprecated_value is not None:
+            old_value = kwargs[old_name]
+            if old_value is not None:
                 warnings.warn(
-                    f"The parameter `{deprecated_name}` of `{node.name}` will be "
+                    f"The parameter `{old_name}` of `{node.name}` will be "
                     f"removed in the future. Please use `{new_name}` instead."
                 )
-                kwargs[new_name] = deprecated_value
+                kwargs[new_name] = old_value
             updated_value = kwargs[new_name]
             if is_required:
                 check_argument(
