@@ -1,16 +1,18 @@
-# Copyright 2021 Q-CTRL. All rights reserved.
+# Copyright 2022 Q-CTRL. All rights reserved.
 #
 # Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
 # copying or use of this file, via any medium, is strictly prohibited.
 # Proprietary and confidential. You may not use this file except in compliance
 # with the License. You may obtain a copy of the License at
 #
-#      https://q-ctrl.com/terms
+#    https://q-ctrl.com/terms
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS. See the
 # License for the specific language.
-"""Module for InfidelityNode."""
+"""
+Module for infidelity related Nodes.
+"""
 from typing import (
     List,
     Optional,
@@ -19,9 +21,10 @@ from typing import (
 
 import forge
 import numpy as np
+from scipy.sparse import spmatrix
 
 from qctrlcommons.node.base import Node
-from qctrlcommons.node.deprecation import deprecated_parameter
+from qctrlcommons.node.documentation import Category
 from qctrlcommons.node.node_data import (
     Pwc,
     Stf,
@@ -29,20 +32,122 @@ from qctrlcommons.node.node_data import (
     Tensor,
 )
 from qctrlcommons.node.utils import (
-    is_broadcastable,
+    TensorLike,
+    get_broadcasted_shape,
     validate_batch_and_value_shapes,
-    validate_broadcasted_shape,
     validate_shape,
 )
 from qctrlcommons.preconditions import (
     check_argument,
+    check_argument_nonzero,
+    check_argument_numeric,
+    check_argument_orthogonal_projection_operator,
+    check_argument_partial_isometry,
+    check_operator,
     check_sample_times,
+    check_square_pwc_or_stf,
 )
 
 
-@deprecated_parameter(  # deprecated 2021/07/14
-    deprecated_name="target_operator", new_name="target"
-)
+class TargetOperation(Node):
+    r"""
+    Creates information about the target for system time evolution.
+
+    Nodes created with this function contain two types of information: the
+    target gate for the system time evolution, and the projection operator
+    that defines the subspace of interest for robustness.
+
+    Parameters
+    ----------
+    operator : np.ndarray or Tensor
+         The target gate :math:`U_\mathrm{target}`. Must be a non-zero partial
+         isometry.
+    filter_function_projector : np.ndarray, optional
+        The orthogonal projection matrix :math:`P` onto the subspace used for
+        filter function calculations. If you provide a value then it must be
+        Hermitian and idempotent. Defaults to the identity matrix.
+
+    Returns
+    -------
+    Target
+        The node containing the specified target information.
+
+    See Also
+    --------
+    infidelity_pwc : Total infidelity of a system with a piecewise-constant Hamiltonian.
+    infidelity_stf : Total infidelity of a system with a sampleable Hamiltonian.
+
+    Notes
+    -----
+    The target gate :math:`U_\mathrm{target}` is a non-zero partial isometry,
+    which means that it can be expressed in the form
+    :math:`\sum_j \left|\psi_j\right>\left<\phi_j\right|`, where
+    :math:`\left\{\left|\psi_j\right>\right\}` and
+    :math:`\left\{\left|\phi_j\right>\right\}` both form (non-empty)
+    orthonormal, but not necessarily complete, sets. Such a target represents
+    a target state :math:`\left|\psi_j\right>` for each initial state
+    :math:`\left|\phi_j\right>`. The resulting operational infidelity is 0
+    if and only if, up to global phase, each initial state
+    :math:`\left|\phi_j\right>` maps exactly to the corresponding final state
+    :math:`\left|\psi_j\right>`.
+
+    The filter function projector :math:`P` is an orthogonal projection
+    matrix, which means that it satisfies :math:`P=P^\dagger=P^2`. The image
+    of :math:`P` defines the set of initial states from which the calculated
+    filter function measures robustness.
+
+    Examples
+    --------
+    Define a target operation for the Pauli :math:`X` gate.
+
+    >>> target_operation = graph.target(operator=np.array([[0, 1], [1, 0]]))
+    >>> target_operation
+    <Target: operation_name="target", value_shape=(2, 2)>
+
+    See more examples in the `How to optimize controls robust to strong noise sources
+    <https://docs.q-ctrl.com/boulder-opal/user-guides/how-to-optimize-controls-robust-
+    to-strong-noise-sources>`_ user guide.
+    """
+
+    name = "target"
+    args = [
+        forge.arg("operator", type=TensorLike),
+        forge.arg("filter_function_projector", type=Optional[np.ndarray], default=None),
+    ]
+    kwargs = {}  # Target should not accept `name` as parameter
+    rtype = Target
+    categories = [Category.OPTIMAL_AND_ROBUST_CONTROL]
+
+    @classmethod
+    def create_node_data(cls, _operation, **kwargs):
+        operator = kwargs.get("operator")
+        value_shape = validate_shape(operator, "operator")
+
+        if isinstance(operator, np.ndarray):
+            check_argument_numeric(operator, "operator")
+            check_argument_partial_isometry(operator, "operator")
+            check_argument_nonzero(operator, "operator")
+        else:
+            check_operator(operator, "operator")
+            check_argument(
+                len(value_shape) == 2,
+                "The operator must be a matrix, not a batch.",
+                {"operator": operator},
+                extras={"operator.shape": value_shape},
+            )
+
+        filter_function_projector = kwargs.get("filter_function_projector")
+        if filter_function_projector is not None:
+            check_argument_numeric(
+                filter_function_projector, "filter_function_projector"
+            )
+            check_argument_orthogonal_projection_operator(
+                filter_function_projector, "filter_function_projector"
+            )
+
+        return Target(_operation, value_shape=value_shape)
+
+
 class InfidelityPwc(Node):
     r"""
     Creates the total infidelity of the given piecewise-constant system.
@@ -141,6 +246,26 @@ class InfidelityPwc(Node):
     is trace-free on the subspace of interest. The filter function value at
     zero frequency quantifies the sensitivity of the controls to quasi-static
     noise applied via the corresponding noise operator.
+
+    Examples
+    --------
+    Calculate infidelity of the identity gate for a noiseless single qubit.
+
+    >>> sigma_z = np.array([[1, 0], [0, -1]])
+    >>> hamiltonian = graph.pwc(
+    ...     durations=np.array([0.1, 0.1]), values=np.array([sigma_z, -sigma_z])
+    ... )
+    >>> target = graph.target(np.eye(2))
+    >>> infidelity = graph.infidelity_pwc(
+    ...     hamiltonian=hamiltonian, target=target, name="infidelity"
+    ... )
+    >>> result = qctrl.functions.calculate_graph(graph=graph, output_node_names=["infidelity"])
+    >>> result.output["infidelity"]["value"]
+    0.0
+
+    See more examples in the `How to optimize controls with non-linear dependencies
+    <https://docs.q-ctrl.com/boulder-opal/user-guides/how-to-optimize-controls-
+    with-nonlinear-dependences>`_ user guide.
     """
 
     name = "infidelity_pwc"
@@ -154,6 +279,7 @@ class InfidelityPwc(Node):
         ),
     ]
     rtype = Tensor
+    categories = [Category.OPTIMAL_AND_ROBUST_CONTROL]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -170,40 +296,45 @@ class InfidelityPwc(Node):
             "The target must be a Target.",
             {"target": target},
         )
-        batch_shape, _ = validate_batch_and_value_shapes(hamiltonian, "hamiltonian")
+        batch_shape, value_shape = validate_batch_and_value_shapes(
+            hamiltonian, "hamiltonian"
+        )
+        check_square_pwc_or_stf(hamiltonian, "hamiltonian")
+        check_argument(
+            value_shape == target.value_shape,
+            "The Hamiltonian and the target must have the same value shape.",
+            {"hamiltonian": hamiltonian, "target": target},
+        )
         for noise_operator in noise_operators or []:
-            check_argument(
-                isinstance(noise_operator, (np.ndarray, Tensor, Pwc)),
-                "Each of the noise_operators must be an array, a Tensor, or a Pwc.",
-                {"noise_operators": noise_operators},
-            )
             if isinstance(noise_operator, Pwc):
-                noise_operator_batch_shape, _ = validate_batch_and_value_shapes(
-                    noise_operator, "noise_operators"
-                )
+                (
+                    noise_operator_batch_shape,
+                    noise_operator_value_shape,
+                ) = validate_batch_and_value_shapes(noise_operator, "noise_operators")
             else:
-                noise_operator_batch_shape = validate_shape(
-                    noise_operator, "noise_operators"
-                )[:-2]
+                check_argument(
+                    not isinstance(noise_operator, (spmatrix, Stf)),
+                    "Noise operator must not be sparse or an Stf.",
+                    {"noise_operators": noise_operators},
+                )
+                check_operator(noise_operator, "noise_operators")
+                noise_operator_shape = validate_shape(noise_operator, "noise_operators")
+                noise_operator_batch_shape = noise_operator_shape[:-2]
+                noise_operator_value_shape = noise_operator_shape[-2:]
             check_argument(
-                is_broadcastable(batch_shape, noise_operator_batch_shape),
+                noise_operator_value_shape == value_shape,
+                "The Hamiltonian and the noise operators must have the same value shape.",
+                {"hamiltonian": hamiltonian, "noise_operators": noise_operators},
+            )
+            batch_shape = get_broadcasted_shape(batch_shape, noise_operator_batch_shape)
+            check_argument(
+                batch_shape is not None,
                 "The batch shapes of the Hamiltonian and noise_operators must be broadcastable.",
                 {"hamiltonian": hamiltonian, "noise_operators": noise_operators},
             )
-
-            # Update the overall batch shape to take into account this noise operator (note that
-            # the check can't fail, since we already ensured the shapes were broadcastable, so we
-            # can pass dummy names).
-            batch_shape = validate_broadcasted_shape(
-                batch_shape, noise_operator_batch_shape, "", ""
-            )
-
         return Tensor(_operation, shape=batch_shape)
 
 
-@deprecated_parameter(  # deprecated 2021/07/14
-    deprecated_name="target_operator", new_name="target"
-)
 class InfidelityStf(Node):
     r"""
     Creates the total infidelity of a given system with a sampleable Hamiltonian.
@@ -249,6 +380,27 @@ class InfidelityStf(Node):
     target : Define the target operation of the time evolution.
     time_evolution_operators_stf : Unitary time evolution operators for quantum systems with
         `Stf` Hamiltonians.
+
+    Examples
+    --------
+    Calculate the infidelity of the Pauli :math:`X` gate for a noiseless qubit.
+
+    >>> sigma_x = np.array([[0, 1], [1, 0]])
+    >>> hamiltonian = graph.constant_stf_operator(np.pi * sigma_x / 2)
+    >>> target = graph.target(sigma_x)
+    >>> infidelity = graph.infidelity_stf(
+    ...     sample_times=np.linspace(0, 0.5, 100),
+    ...     hamiltonian=hamiltonian,
+    ...     target=target,
+    ...     name="infidelity",
+    ... )
+    >>> result = qctrl.functions.calculate_graph(graph=graph, output_node_names=["infidelity"])
+    >>> result.output["infidelity"]["value"]
+    0.5000000000260991
+
+    See more examples in the `How to perform model-based optimization with user-defined
+    basis functions <https://docs.q-ctrl.com/boulder-opal/user-guides/how-to-perform-model-
+    based-optimization-with-user-defined-basis-functions>`_ user guide.
     """
 
     name = "infidelity_stf"
@@ -263,6 +415,7 @@ class InfidelityStf(Node):
         ),
     ]
     rtype = Tensor
+    categories = [Category.OPTIMAL_AND_ROBUST_CONTROL]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -286,32 +439,41 @@ class InfidelityStf(Node):
             "The target must be a Target.",
             {"target": target},
         )
-        batch_shape, _ = validate_batch_and_value_shapes(hamiltonian, "hamiltonian")
+        batch_shape, value_shape = validate_batch_and_value_shapes(
+            hamiltonian, "hamiltonian"
+        )
+        check_square_pwc_or_stf(hamiltonian, "hamiltonian")
+        check_argument(
+            value_shape == target.value_shape,
+            "The Hamiltonian and the target must have the same value shape.",
+            {"hamiltonian": hamiltonian, "target": target},
+        )
         for noise_operator in noise_operators or []:
-            check_argument(
-                isinstance(noise_operator, (np.ndarray, Tensor, Stf)),
-                "Each of the noise_operators must be an array, a Tensor, or an Stf.",
-                {"noise_operators": noise_operators},
-            )
             if isinstance(noise_operator, Stf):
-                noise_operator_batch_shape, _ = validate_batch_and_value_shapes(
-                    noise_operator, "noise_operators"
-                )
+                (
+                    noise_operator_batch_shape,
+                    noise_operator_value_shape,
+                ) = validate_batch_and_value_shapes(noise_operator, "noise_operators")
             else:
-                noise_operator_batch_shape = validate_shape(
-                    noise_operator, "noise_operators"
-                )[:-2]
+                check_argument(
+                    not isinstance(noise_operator, (spmatrix, Pwc)),
+                    "Noise operator must not be sparse or a Pwc.",
+                    {"noise_operators": noise_operators},
+                )
+                check_operator(noise_operator, "noise_operators")
+                noise_operator_shape = validate_shape(noise_operator, "noise_operators")
+                noise_operator_batch_shape = noise_operator_shape[:-2]
+                noise_operator_value_shape = noise_operator_shape[-2:]
             check_argument(
-                is_broadcastable(batch_shape, noise_operator_batch_shape),
+                noise_operator_value_shape == value_shape,
+                "The Hamiltonian and the noise operators must have the same value shape.",
+                {"hamiltonian": hamiltonian, "noise_operators": noise_operators},
+            )
+            batch_shape = get_broadcasted_shape(batch_shape, noise_operator_batch_shape)
+            check_argument(
+                batch_shape is not None,
                 "The batch shapes of the Hamiltonian and noise_operators must be broadcastable.",
                 {"hamiltonian": hamiltonian, "noise_operators": noise_operators},
             )
 
-            # Update the overall batch shape to take into account this noise operator (note that
-            # the check can't fail, since we already ensured the shapes were broadcastable, so we
-            # can pass dummy names).
-            batch_shape = validate_broadcasted_shape(
-                batch_shape, noise_operator_batch_shape, "", ""
-            )
-
         return Tensor(_operation, shape=batch_shape)
