@@ -1,11 +1,11 @@
-# Copyright 2021 Q-CTRL. All rights reserved.
+# Copyright 2022 Q-CTRL. All rights reserved.
 #
 # Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
 # copying or use of this file, via any medium, is strictly prohibited.
 # Proprietary and confidential. You may not use this file except in compliance
 # with the License. You may obtain a copy of the License at
 #
-#      https://q-ctrl.com/terms
+#    https://q-ctrl.com/terms
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS. See the
@@ -114,7 +114,7 @@ class Tensor(NamedNodeData, ArithmeticMixin):
         The name assigned to the node.
     """
 
-    shape: Tuple[int]
+    shape: Tuple[int, ...]
 
     def __getitem__(self, item) -> "Operation":
         """
@@ -173,12 +173,12 @@ class Pwc(NamedNodeData, ArithmeticMixin):
 
     See Also
     --------
-    pwc : Create piecewise-constant functions.
+    ~Graph.pwc : Operation to create piecewise-constant functions.
     """
 
-    value_shape: Tuple[int]
+    value_shape: Tuple[int, ...]
     durations: np.ndarray
-    batch_shape: Tuple[int]
+    batch_shape: Tuple[int, ...]
 
     @property
     def values(self):
@@ -223,11 +223,11 @@ class Stf(NodeData, ArithmeticMixin):
 
     See Also
     --------
-    identity_stf : Create an `Stf` representing the identity function.
+    ~Graph.identity_stf : Operation to create an `Stf` representing the identity function.
     """
 
-    value_shape: Tuple[int]
-    batch_shape: Tuple[int]
+    value_shape: Tuple[int, ...]
+    batch_shape: Tuple[int, ...]
 
     def __repr__(self):
         return (
@@ -248,10 +248,10 @@ class SparsePwc(NodeData):
 
     See Also
     --------
-    sparse_pwc_operator : Create `SparsePwc` operators.
+    ~Graph.sparse_pwc_operator : Operation to create `SparsePwc` operators.
     """
 
-    value_shape: Tuple[int]
+    value_shape: Tuple[int, ...]
 
     def __repr__(self):
         return (
@@ -272,10 +272,10 @@ class Target(NodeData):
 
     See Also
     --------
-    target : Define the target operation of a time evolution.
+    ~Graph.target : Operation to define the target operation of a time evolution.
     """
 
-    value_shape: Tuple[int]
+    value_shape: Tuple[int, ...]
 
     def __repr__(self):
         return (
@@ -291,9 +291,11 @@ class ConvolutionKernel(NodeData):
 
     See Also
     --------
-    convolve_pwc : Create an `Stf` by convolving a `Pwc` with a kernel.
-    gaussian_convolution_kernel : Create a convolution kernel representing a normalized Gaussian.
-    sinc_convolution_kernel : Create a convolution kernel representing the sinc function.
+    ~Graph.convolve_pwc : Operation to create an `Stf` by convolving a `Pwc` with a kernel.
+    ~Graph.gaussian_convolution_kernel : Operation to create a convolution kernel representing a
+        normalized Gaussian.
+    ~Graph.sinc_convolution_kernel : Operation to create a convolution kernel representing the sinc
+        function.
     """
 
     def __repr__(self):
