@@ -1,19 +1,20 @@
-# Copyright 2021 Q-CTRL. All rights reserved.
+# Copyright 2022 Q-CTRL. All rights reserved.
 #
 # Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
 # copying or use of this file, via any medium, is strictly prohibited.
 # Proprietary and confidential. You may not use this file except in compliance
 # with the License. You may obtain a copy of the License at
 #
-#      https://q-ctrl.com/terms
+#    https://q-ctrl.com/terms
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS. See the
 # License for the specific language.
-"""Module for all the node related to PWC."""
+"""
+Module for all the node related to PWC.
+"""
 from typing import (
     List,
-    Optional,
     Union,
 )
 
@@ -22,20 +23,22 @@ import numpy as np
 
 from qctrlcommons.node.base import Node
 from qctrlcommons.node.deprecation import deprecated_node
+from qctrlcommons.node.documentation import Category
 from qctrlcommons.node.node_data import (
     Pwc,
     Tensor,
 )
 from qctrlcommons.node.utils import (
+    TensorLike,
+    get_broadcasted_shape,
     validate_batch_and_value_shapes,
     validate_hamiltonian,
-    validate_inputs_real_fourier_signal,
     validate_shape,
 )
 from qctrlcommons.preconditions import (
     check_argument,
     check_argument_integer,
-    check_argument_iteratable,
+    check_argument_iterable,
     check_argument_numeric,
     check_duration,
     check_operator,
@@ -85,16 +88,42 @@ class PwcOperation(Node):
     pwc_operator : Create `Pwc` operators.
     pwc_signal : Create `Pwc` signals from (possibly complex) values.
     pwc_sum : Sum multiple `Pwc`\s.
+
+    Examples
+    --------
+    Create a Hamiltonian from a piecewise-constant signal with non-uniform segment durations.
+
+    >>> omega = graph.pwc(
+    ...     values=np.array([1, 2, 3]), durations=np.array([0.1, 0.2, 0.3]), name="omega"
+    ... )
+    >>> omega
+    <Pwc: name="omega", operation_name="pwc", value_shape=(), batch_shape=()>
+    >>> sigma_z = np.array([[1, 0], [0, -1]])
+    >>> hamiltonian = omega * sigma_z
+    >>> hamiltonian.name = "hamiltonian"
+    >>> result = qctrl.functions.calculate_graph(graph=graph, output_node_names=["hamiltonian"])
+    >>> result.output["hamiltonian"]
+    [
+        {"value": array([[1.0, 0.0], [0.0, -1.0]]), "duration": 0.1},
+        {"value": array([[2.0, 0.0], [0.0, -2.0]]), "duration": 0.2},
+        {"value": array([[3.0, 0.0], [0.0, -3.0]]), "duration": 0.3},
+    ]
+
+    See more examples in the `How to simulate quantum dynamics subject to noise with graphs
+    <https://docs.q-ctrl.com/boulder-opal/user-guides/how-to-simulate-
+    quantum-dynamics-subject-to-noise-with-graphs>`_ user guide.
+
     """
 
     name = "pwc"
     args = [
         forge.arg("durations", type=np.ndarray),
-        forge.arg("values", type=Union[np.ndarray, Tensor]),
+        forge.arg("values", type=TensorLike),
         forge.arg("time_dimension", type=int, default=0),
     ]
 
     rtype = Pwc
+    categories = [Category.BUILDING_PIECEWISE_CONSTANT_HAMILTONIANS]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -138,13 +167,6 @@ class PwcOperation(Node):
         )
 
 
-@deprecated_node("pwc")  # deprecated 2021/05/20
-class TensorPwc(PwcOperation):
-    """Deprecated node."""
-
-    name = "tensor_pwc"
-
-
 class PwcSignal(Node):
     r"""
     Creates a piecewise-constant signal (scalar-valued function of time).
@@ -183,14 +205,25 @@ class PwcSignal(Node):
     pwc_operator : Create `Pwc` operators.
     pwc_sum : Sum multiple `Pwc`\s.
     symmetrize_pwc : Symmetrize `Pwc`\s.
+
+    Examples
+    --------
+    Create a piecewise-constant signal with uniform segment duration.
+
+    >>> graph.pwc_signal(duration=0.1, values=np.array([2, 3]), name="signal")
+    <Pwc: name="signal", operation_name="pwc_signal", value_shape=(), batch_shape=()>
+    >>> result = qctrl.functions.calculate_graph(graph=graph, output_node_names=["signal"])
+    >>> result.output["signal"]
+    [{'value': 2.0, 'duration': 0.05}, {'value': 3.0, 'duration': 0.05}]
+
+    See more examples in the `Get familiar with graphs <https://docs.q-ctrl.com/
+    boulder-opal/tutorials/get-familiar-with-graphs>`_ tutorial.
     """
 
     name = "pwc_signal"
-    args = [
-        forge.arg("values", type=Union[np.ndarray, Tensor]),
-        forge.arg("duration", type=float),
-    ]
+    args = [forge.arg("values", type=TensorLike), forge.arg("duration", type=float)]
     rtype = Pwc
+    categories = [Category.BUILDING_PIECEWISE_CONSTANT_HAMILTONIANS]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -206,10 +239,7 @@ class PwcSignal(Node):
         )
         durations = duration / shape[-1] * np.ones(shape[-1])
         return Pwc(
-            _operation,
-            value_shape=(),
-            durations=durations,
-            batch_shape=shape[:-1],
+            _operation, value_shape=(), durations=durations, batch_shape=shape[:-1]
         )
 
 
@@ -253,15 +283,41 @@ class ComplexPwcSignal(Node):
     See Also
     --------
     pwc_signal : Create `Pwc` signals from (possibly complex) values.
+
+    Examples
+    --------
+    Create a complex piecewise-constant signal with batched moduli and phases.
+
+    >>> moduli = np.array([[1, 2], [3, 4]])
+    >>> phases = np.array([[0.1, 0.2], [0.5, 0.7]])
+    >>> graph.complex_pwc_signal(moduli=moduli, phases=phases, duration=0.2, name="signal")
+    <Pwc: name="signal", operation_name="complex_pwc_signal", value_shape=(), batch_shape=(2,)>
+    >>> result = qctrl.functions.calculate_graph(graph=graph, output_node_names=["signal"])
+    >>> result.output["signal"]
+    [
+        [
+            {"value": (0.9950041652780258 + 0.09983341664682815j), "duration": 0.1},
+            {"value": (1.9601331556824833 + 0.39733866159012243j), "duration": 0.1},
+        ],
+        [
+            {"value": (2.6327476856711183 + 1.438276615812609j), "duration": 0.1},
+            {"value": (3.059368749137954 + 2.576870748950764j), "duration": 0.1},
+        ],
+    ]
+
+    See more examples in the `Design robust single-qubit gates using computational graphs
+    <https://docs.q-ctrl.com/boulder-opal/tutorials/design-robust-single-qubit-gates-
+    using-computational-graphs>`_ tutorial.
     """
 
     name = "complex_pwc_signal"
     args = [
-        forge.arg("moduli", type=Union[np.ndarray, Tensor]),
-        forge.arg("phases", type=Union[np.ndarray, Tensor]),
+        forge.arg("moduli", type=TensorLike),
+        forge.arg("phases", type=TensorLike),
         forge.arg("duration", type=float),
     ]
     rtype = Pwc
+    categories = [Category.BUILDING_PIECEWISE_CONSTANT_HAMILTONIANS]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -317,29 +373,46 @@ class PwcOperator(Node):
     --------
     complex_pwc_signal : Create complex `Pwc` signals from their moduli and phases.
     constant_pwc_operator : Create constant `Pwc`\s.
+    hermitian_part : Hermitian part of an operator.
     pwc : Create piecewise-constant functions.
-    pwc_operator_hermitian_part : Hermitian part of a piecewise-constant operator.
     pwc_signal : Create `Pwc` signals from (possibly complex) values.
     pwc_sum : Sum multiple `Pwc`\s.
     stf_operator : Corresponding operation for `Stf`\s.
-    stf_operator_hermitian_part : Corresponding operation for `Stf`\s.
+
+    Examples
+    --------
+    Create a piecewise-constant operator with non-uniform segment durations.
+
+    >>> sigma_z = np.array([[1.0, 0.0],[0.0, -1.0]])
+    >>> graph.pwc_operator(
+    ...     signal=graph.pwc(durations=np.array([0.1, 0.2]), values=np.array([1, 2])),
+    ...     operator=sigma_z,
+    ...     name="operator",
+    ... )
+    <Pwc: name="operator", operation_name="pwc_operator", value_shape=(2, 2), batch_shape=()>
+    >>> result = qctrl.functions.calculate_graph(graph=graph, output_node_names=["operator"])
+    >>> result.operator["operator"]
+    [
+        {"value": array([[1.0, 0.0], [0.0, -1.0]]), "duration": 0.1},
+        {"value": array([[2.0, 0.0], [0.0, -2.0]]), "duration": 0.2},
+    ]
+
+    See more examples in the `How to represent quantum systems using graphs
+    <https://docs.q-ctrl.com/boulder-opal/user-guides/how-to-represent-quantum-
+    systems-using-graphs>`_ user guide.
     """
 
     name = "pwc_operator"
-    args = [
-        forge.arg("signal", type=Pwc),
-        forge.arg("operator", type=Union[np.ndarray, Tensor]),
-    ]
+    args = [forge.arg("signal", type=Pwc), forge.arg("operator", type=TensorLike)]
     rtype = Pwc
+    categories = [Category.BUILDING_PIECEWISE_CONSTANT_HAMILTONIANS]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
         signal = kwargs.get("signal")
         operator = kwargs.get("operator")
         check_argument(
-            isinstance(signal, Pwc),
-            "The signal must be a Pwc.",
-            {"signal": signal},
+            isinstance(signal, Pwc), "The signal must be a Pwc.", {"signal": signal}
         )
         batch_shape, signal_value_shape = validate_batch_and_value_shapes(
             signal, "signal"
@@ -390,16 +463,34 @@ class ConstantPwcOperator(Node):
     See Also
     --------
     constant_stf_operator : Corresponding operation for `Stf`\s.
+    hermitian_part : Hermitian part of an operator.
     pwc_operator : Create `Pwc` operators.
-    pwc_operator_hermitian_part : Hermitian part of a piecewise-constant operator.
+
+    Examples
+    --------
+    Create a Hamiltonian from a batched constant operator.
+
+    >>> sigma_x = np.array([[0.0, 1.0], [1.0, 0.0]])
+    >>> sigma_z = np.array([[1.0, 0.0], [0.0, -1.0]])
+    >>> batched_operators = np.asarray([sigma_x, sigma_z])
+    >>> graph.constant_pwc_operator(duration=0.1, operator=batched_operators,  name="op")
+    <Pwc: name="op", operation_name="constant_pwc_operator", value_shape=(2, 2), batch_shape=(2,)>
+    >>> result = qctrl.functions.calculate_graph(graph=graph, output_node_names=["op"])
+    >>> result.output["op"]
+    [
+        [{"value": array([[0.0, 1.0], [1.0, 0.0]]), "duration": 0.1}],
+        [{"value": array([[1.0, 0.0], [0.0, -1.0]]), "duration": 0.1}],
+    ]
+
+    See more examples in the `How to represent quantum systems using graphs
+    <https://docs.q-ctrl.com/boulder-opal/user-guides/how-to-represent-quantum-
+    systems-using-graphs>`_ user guide.
     """
 
     name = "constant_pwc_operator"
-    args = [
-        forge.arg("duration", type=float),
-        forge.arg("operator", type=Union[np.ndarray, Tensor]),
-    ]
+    args = [forge.arg("duration", type=float), forge.arg("operator", type=TensorLike)]
     rtype = Pwc
+    categories = [Category.BUILDING_PIECEWISE_CONSTANT_HAMILTONIANS]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -418,7 +509,7 @@ class ConstantPwcOperator(Node):
 
 class ConstantPwc(Node):
     r"""
-    Creates a piecewise-constant (PWC) function of time that is constant over a specified duration.
+    Creates a piecewise-constant function of time that is constant over a specified duration.
 
     Parameters
     ----------
@@ -448,6 +539,26 @@ class ConstantPwc(Node):
     constant_pwc_operator : Create constant `Pwc` operators.
     constant_stf : Corresponding operation for `Stf`\s.
     pwc : Create piecewise-constant functions.
+
+    Examples
+    --------
+    Create a batched piecewise-constant function.
+
+    >>> constant = np.arange(12).reshape((2, 2, 3))
+    >>> graph.constant_pwc(
+    ...     constant=constant, duration=0.1, batch_dimension_count=1, name="constant"
+    ... )
+    <Pwc: name="constant", operation_name="constant_pwc", value_shape=(2, 3), batch_shape=(2,)>
+    >>> result = qctrl.functions.calculate_graph(graph=graph, output_node_names=["constant"])
+    >>> result.output["constant"]
+    [
+        [{"value": array([[0.0, 1.0, 2.0], [3.0, 4.0, 5.0]]), "duration": 0.1}],
+        [{"value": array([[6.0, 7.0, 8.0], [9.0, 10.0, 11.0]]), "duration": 0.1}],
+    ]
+
+    See more examples in the `Simulate the dynamics of a single qubit using computational graphs
+    <https://docs.q-ctrl.com/boulder-opal/tutorials/simulate-the-dynamics-of-a-single-qubit-
+    using-computational-graphs>`_ tutorial.
     """
 
     name = "constant_pwc"
@@ -457,6 +568,7 @@ class ConstantPwc(Node):
         forge.arg("batch_dimension_count", type=int, default=0),
     ]
     rtype = Pwc
+    categories = [Category.BUILDING_PIECEWISE_CONSTANT_HAMILTONIANS]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -474,7 +586,7 @@ class ConstantPwc(Node):
             "The number of batch dimensions must not be larger than the number "
             "of dimensions of the input constant.",
             {"constant": constant, "batch_dimension_count": batch_dimension_count},
-            {"Number of value dimensions": len(shape)},
+            extras={"Number of value dimensions": len(shape)},
         )
         check_argument(
             batch_dimension_count >= 0,
@@ -490,7 +602,9 @@ class ConstantPwc(Node):
         )
 
 
+@deprecated_node(updated_node_name="hermitian_part")  # deprecated 2022/04/21
 class PwcOperatorHermitianPart(Node):
+    # pylint:disable=line-too-long
     r"""
     Creates the Hermitian part of a piecewise-constant operator.
 
@@ -511,13 +625,34 @@ class PwcOperatorHermitianPart(Node):
     --------
     constant_pwc_operator : Create constant `Pwc`\s.
     pwc_operator : Create `Pwc` operators.
+    sparse_pwc_hermitian_part : Create the Hermitian part of a `SparsePwc` operator.
+
+    Examples
+    --------
+    Creates a Hamiltonian from a non-Hermitian operator.
+
+    >>> omega = graph.pwc(durations=np.array([0.1, 0.2]), values=np.array([0.2, 0.4]))
+    >>> sigma_m = np.array([[0, 1], [0, 0]])
+    >>> operator = omega * sigma_m
+    >>> graph.pwc_operator_hermitian_part(operator=operator, name="hamiltonian")
+    <Pwc: name="hamiltonian", operation_name="pwc_operator_hermitian_part", value_shape=(2, 2), batch_shape=()>
+    >>> result = qctrl.functions.calculate_graph(graph=graph, output_node_names=["hamiltonian"])
+    >>> result.output["hamiltonian"]
+    [
+        {"value": array([[0.0, 0.1], [0.1, 0.0]]), "duration": 0.1},
+        {"value": array([[0.0, 0.2], [0.2, 0.0]]), "duration": 0.2},
+    ]
+
+    See more examples in the `Simulate the dynamics of a single qubit using computational graphs
+    <https://docs.q-ctrl.com/boulder-opal/tutorials/simulate-the-dynamics-of-a-single-qubit-
+    using-computational-graphs>`_ tutorial.
     """
 
+    # pylint:enable=line-too-long
     name = "pwc_operator_hermitian_part"
-    args = [
-        forge.arg("operator", type=Pwc),
-    ]
+    args = [forge.arg("operator", type=Pwc)]
     rtype = Pwc
+    categories = [Category.BUILDING_PIECEWISE_CONSTANT_HAMILTONIANS]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -527,9 +662,11 @@ class PwcOperatorHermitianPart(Node):
             "The operator must be a Pwc.",
             {"operator": operator},
         )
-        batch_shape, value_shape = validate_batch_and_value_shapes(
-            operator,
-            "operator",
+        batch_shape, value_shape = validate_batch_and_value_shapes(operator, "operator")
+        check_argument(
+            len(value_shape) == 2 and value_shape[0] == value_shape[1],
+            "The values of the operator must be a square matrix.",
+            {"operator": operator},
         )
         return Pwc(
             _operation,
@@ -571,25 +708,46 @@ class PwcSum(Node):
     pwc_operator : Create `Pwc` operators.
     pwc_signal : Create `Pwc` signals from (possibly complex) values.
     stf_sum : Corresponding operation for `Stf`\s.
+
+    Examples
+    --------
+    Sum a list of piecewise-constant terms of different durations.
+
+    >>> x = graph.pwc(durations=np.array([0.1, 0.3]), values=np.array([1, 2]))
+    >>> y = graph.pwc(durations=np.array([0.2, 0.2]), values=np.array([3, 1]))
+    >>> graph.pwc_sum([x, y], name="sum")
+    <Pwc: name="sum", operation_name="pwc_sum", value_shape=(), batch_shape=()>
+    >>> result = qctrl.functions.calculate_graph(graph=graph, output_node_names=["sum"])
+    >>> result.output["sum"]
+    [
+        {"value": 4.0, "duration": 0.1},
+        {"value": 5.0, "duration": 0.1},
+        {"value": 3.0, "duration": 0.2},
+    ]
+
+    See more examples in the `How to optimize controls robust to strong noise sources
+    <https://docs.q-ctrl.com/boulder-opal/user-guides/how-to-optimize-controls-robust-to-
+    strong-noise-sources>`_ user guide.
     """
 
     name = "pwc_sum"
     args = [forge.arg("terms", type=List[Pwc])]
     rtype = Pwc
+    categories = [Category.BUILDING_PIECEWISE_CONSTANT_HAMILTONIANS]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
         terms = kwargs.get("terms")
-        check_argument_iteratable(terms, "terms")
+        check_argument_iterable(terms, "terms")
+        check_argument(
+            len(terms) > 0, "You must provide at least one term.", {"terms": terms}
+        )
         check_argument(
             all(isinstance(term, Pwc) for term in terms),
             "Each of the terms must be a Pwc.",
             {"terms": terms},
         )
-        batch_shape, value_shape = validate_batch_and_value_shapes(
-            terms[0],
-            "terms[0]",
-        )
+        batch_shape, value_shape = validate_batch_and_value_shapes(terms[0], "terms[0]")
         check_argument(
             all(
                 (
@@ -631,135 +789,61 @@ class PwcSum(Node):
         )
 
 
-class RealFourierPwcSignal(Node):
+class TimeReversePwc(Node):
     r"""
-    Creates a piecewise-constant signal constructed from Fourier components.
-
-    Use this function to create a signal defined in terms of Fourier
-    (sine/cosine) basis signals that can be optimized by varying their
-    coefficients and, optionally, their frequencies.
+    Reverses in time a piecewise-constant function.
 
     Parameters
     ----------
-    duration : float
-        The total duration :math:`\tau` of the signal.
-    segment_count : int
-        The number of segments :math:`N` to use for the piecewise-constant
-        approximation.
-    initial_coefficient_lower_bound : float, optional
-        The lower bound :math:`c_\mathrm{min}` on the initial coefficient
-        values. Defaults to -1.
-    initial_coefficient_upper_bound : float, optional
-        The upper bound :math:`c_\mathrm{max}` on the initial coefficient
-        values. Defaults to 1.
-    fixed_frequencies : list[float], optional
-        The fixed non-zero frequencies :math:`\{f_m\}` to use for the Fourier
-        basis. Must be non-empty if provided. Must be specified in the inverse
-        units of `duration` (for example if `duration` is in seconds, these
-        values must be given in Hertz).
-    optimizable_frequency_count : int, optional
-        The number of non-zero frequencies :math:`M` to use, if the
-        frequencies can be optimized. Must be greater than zero if provided.
-    randomized_frequency_count : int, optional
-        The number of non-zero frequencies :math:`M` to use, if the
-        frequencies are to be randomized but fixed. Must be greater than zero
-        if provided.
+    pwc : Pwc
+        The piecewise-constant function :math:`v(t)` to reverse.
     name : str, optional
         The name of the node.
 
     Returns
     -------
-    Pwc(1D, real)
-        The optimizable, real-valued, piecewise-constant signal built from the
-        appropriate Fourier components.
-
-    Warnings
-    --------
-    You must provide exactly one of `fixed_frequencies`, `optimizable_variable`,
-    or `randomized_frequency_count`.
+    Pwc
+        The piecewise-constant function :math:`w(t)` defined by
+        :math:`w(t)=v(\tau-t)` for :math:`0\leq t\leq \tau`, where
+        :math:`\tau` is the duration of :math:`v(t)`.
 
     See Also
     --------
-    real_fourier_stf_signal : Corresponding operation for `Stf`.
+    symmetrize_pwc : Symmetrize `Pwc`\s.
+    time_concatenate_pwc : Concatenate `Pwc`\s in time.
 
-    Notes
-    -----
-    This function sets the basis signal frequencies :math:`\{f_m\}`
-    depending on the chosen mode:
+    Examples
+    --------
+    Reverse a piecewise constant function.
 
-    * For fixed frequencies, you provide the frequencies directly.
-    * For optimizable frequencies, you provide the number of frequencies
-      :math:`M`, and this function creates :math:`M` unbounded optimizable
-      variables :math:`\{f_m\}`, with initial values in the ranges
-      :math:`\{[(m-1)/\tau, m/\tau]\}`.
-    * For randomized frequencies, you provide the number of frequencies
-      :math:`M`, and this function creates :math:`M` randomized constants
-      :math:`\{f_m\}` in the ranges :math:`\{[(m-1)/\tau, m/\tau]\}`.
-
-    After this function creates the :math:`M` frequencies :math:`\{f_m\}`, it
-    produces the signal
-
-    .. math::
-        \alpha^\prime(t) = v_0 +
-        \sum_{m=1}^M [ v_m \cos(2\pi t f_m) + w_m \sin(2\pi t f_m) ],
-
-    where :math:`\{v_m,w_m\}` are (unbounded) optimizable variables, with
-    initial values bounded by :math:`c_\mathrm{min}` and
-    :math:`c_\mathrm{max}`. This function produces the final
-    piecewise-constant signal :math:`\alpha(t)` by sampling
-    :math:`\alpha^\prime(t)` at :math:`N` equally spaced points along the
-    duration :math:`\tau`, and using those sampled values as the constant
-    segment values.
-
-    You can use the signals created by this function for chopped random basis
-    (CRAB) optimization [1]_.
-
-    References
-    ----------
-    .. [1] `P. Doria, T. Calarco, and S. Montangero, Physical Review Letters 106, 190501 (2011).
-           <https://doi.org/10.1103/PhysRevLett.106.190501>`_
+    >>> x = graph.pwc(durations=np.array([0.1, 0.5, 0.3]), values=np.array([1, 2, 3]))
+    >>> graph.time_reverse_pwc(x, name="reverse")
+    <Pwc: name="reverse", operation_name="time_reverse_pwc", value_shape=(), batch_shape=()>
+    >>> result = qctrl.functions.calculate_graph(graph=graph, output_node_names=["reverse"])
+    >>> result.output["reverse"]
+    [{'duration': 0.3, 'value': 3.0},
+    {'duration': 0.5, 'value': 2.0},
+    {'duration': 0.1, 'value': 1.0}]
     """
 
-    name = "real_fourier_pwc_signal"
-    optimizable_variable = True
-    args = [
-        forge.arg("duration", type=float),
-        forge.arg("segment_count", type=int),
-        forge.arg("initial_coefficient_lower_bound", type=float, default=-1),
-        forge.arg("initial_coefficient_upper_bound", type=float, default=1),
-        forge.arg("fixed_frequencies", type=Optional[List[float]], default=None),
-        forge.arg("optimizable_frequency_count", type=Optional[int], default=None),
-        forge.arg("randomized_frequency_count", type=Optional[int], default=None),
-    ]
+    name = "time_reverse_pwc"
+    args = [forge.arg("pwc", type=Pwc)]
     rtype = Pwc
+    categories = [Category.BUILDING_PIECEWISE_CONSTANT_HAMILTONIANS]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
-        duration = kwargs.get("duration")
-        segment_count = kwargs.get("segment_count")
-        check_duration(duration, "duration")
-
-        check_argument_integer(segment_count, "segment_count")
+        pwc = kwargs.get("pwc")
         check_argument(
-            segment_count > 0,
-            "The number of segments must be greater than zero.",
-            {"segment_count": segment_count},
+            isinstance(pwc, Pwc), "The pwc parameter must be a Pwc.", {"pwc": pwc}
         )
-
-        fixed_frequencies = kwargs.get("fixed_frequencies")
-        optimizable_frequency_count = kwargs.get("optimizable_frequency_count")
-        randomized_frequency_count = kwargs.get("randomized_frequency_count")
-        validate_inputs_real_fourier_signal(
-            fixed_frequencies, optimizable_frequency_count, randomized_frequency_count
-        )
-
-        durations = duration / segment_count * np.ones(segment_count)
-
+        batch_shape, value_shape = validate_batch_and_value_shapes(pwc, "pwc")
+        reversed_durations = pwc.durations[::-1]
         return Pwc(
             _operation,
-            value_shape=(),
-            durations=durations,
-            batch_shape=(),
+            value_shape=value_shape,
+            durations=reversed_durations,
+            batch_shape=batch_shape,
         )
 
 
@@ -785,24 +869,41 @@ class SymmetrizePwc(Node):
     See Also
     --------
     pwc_signal : Create `Pwc` signals from (possibly complex) values.
+    time_reverse_pwc : Reverse `Pwc`\s in time.
+
+    Examples
+    --------
+    Create a symmetric piecewise-constant function.
+
+    >>> x = graph.pwc(durations=np.array([0.1, 0.3]), values=np.array([1, 2]))
+    >>> graph.symmetrize_pwc(x, name="symmetrize")
+    <Pwc: name="symmetrize", operation_name="symmetrize_pwc", value_shape=(), batch_shape=()>
+    >>> result = qctrl.functions.calculate_graph(graph=graph, output_node_names=["symmetrize"])
+    >>> result.output["symmetrize"]
+    [
+        {"value": 1.0, "duration": 0.1},
+        {"value": 2.0, "duration": 0.3},
+        {"value": 2.0, "duration": 0.3},
+        {"value": 1.0, "duration": 0.1},
+    ]
+
+    See more examples in the `How to optimize controls with time symmetrization
+    <https://docs.q-ctrl.com/boulder-opal/user-guides/how-to-optimize-controls-with-
+    time-symmetrization>`_ user guide.
     """
 
     name = "symmetrize_pwc"
     args = [forge.arg("pwc", type=Pwc)]
     rtype = Pwc
+    categories = [Category.BUILDING_PIECEWISE_CONSTANT_HAMILTONIANS]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
         pwc = kwargs.get("pwc")
         check_argument(
-            isinstance(pwc, Pwc),
-            "The pwc parameter must be a Pwc.",
-            {"pwc": pwc},
-        )
-        batch_shape, value_shape = validate_batch_and_value_shapes(
-            pwc,
-            "pwc",
+            isinstance(pwc, Pwc), "The pwc parameter must be a Pwc.", {"pwc": pwc}
         )
+        batch_shape, value_shape = validate_batch_and_value_shapes(pwc, "pwc")
         symmetrized_durations = np.concatenate((pwc.durations, pwc.durations[::-1]))
         return Pwc(
             _operation,
@@ -812,6 +913,105 @@ class SymmetrizePwc(Node):
         )
 
 
+class TimeConcatenatePwc(Node):
+    r"""
+    Concatenates multiple piecewise-constant functions in the time dimension.
+
+    Parameters
+    ----------
+    pwc_list : list[Pwc]
+        The individual piecewise-constant functions :math:`\{A_i(t)\}` to concatenate.
+        All the functions must have the same value shape, and can have broadcastable batch shapes.
+    name : str, optional
+        The name of the node.
+
+    Returns
+    -------
+    Pwc
+        The concatenated piecewise-constant function (or batch of functions).
+
+    Notes
+    -----
+    The function resulting from the concatenation is
+
+    .. math::
+        C(t) = \begin{cases}
+        A_0(t) & \mathrm{for} & 0 < t < \tau_0
+        \\
+        A_1(t - \tau_0) & \mathrm{for} & \tau_0 < t < \tau_0 + \tau_1
+        \\
+        A_2(t - \tau_0 - \tau_1) & \mathrm{for} & \tau_0 + \tau_1 < t < \tau_0 + \tau_1 + \tau_2
+        \\
+        & \vdots &
+        \end{cases}
+
+    where :math:`\tau_i` is the duration of the i-th function.
+
+    See Also
+    --------
+    pwc : Create piecewise-constant functions.
+    pwc_sum : Sum multiple `Pwc`\s.
+    symmetrize_pwc : Symmetrize `Pwc`\s.
+    time_reverse_pwc : Reverse `Pwc`\s in time.
+
+    Examples
+    --------
+    Concatenate two piecewise-constant functions.
+
+    >>> pwc1 = graph.pwc(durations=np.array([0.2, 0.5]), values=np.array([1, 2]))
+    >>> pwc2 = graph.pwc(durations=np.array([0.7, 0.9]), values=np.array([3, 4]))
+    >>> graph.time_concatenate_pwc([pwc1, pwc2], name="concat")
+    <Pwc: name="concat", operation_name="time_concatenate_pwc", value_shape=(), batch_shape=()>
+    >>> result = qctrl.functions.calculate_graph(graph=graph, output_node_names=["concat"])
+    >>> result.output["concat"]
+    [
+        {'value': 1.0, 'duration': 0.2},
+        {'value': 2.0, 'duration': 0.5},
+        {'value': 3.0, 'duration': 0.7},
+        {'value': 4.0, 'duration': 0.9},
+    ]
+    """
+
+    name = "time_concatenate_pwc"
+    args = [forge.arg("pwc_list", type=List[Pwc])]
+    rtype = Pwc
+    categories = [Category.BUILDING_PIECEWISE_CONSTANT_HAMILTONIANS]
+
+    @classmethod
+    def create_node_data(cls, _operation, **kwargs):
+        pwc_list = kwargs.get("pwc_list")
+        check_argument_iterable(pwc_list, "pwc_list")
+        check_argument(
+            len(pwc_list) > 0,
+            "You must provide at least one element.",
+            {"pwc_list": pwc_list},
+        )
+        check_argument(
+            all(isinstance(pwc, Pwc) for pwc in pwc_list),
+            "Each of the elements to concatenate must be a Pwc.",
+            {"pwc_list": pwc_list},
+        )
+        _, value_shape = validate_batch_and_value_shapes(pwc_list[0], "pwc_list[0]")
+        check_argument(
+            all((pwc.value_shape == value_shape) for pwc in pwc_list[1:]),
+            "All the Pwcs must have the same value shape.",
+            {"pwc_list": pwc_list},
+        )
+        batch_shape = get_broadcasted_shape(*[pwc.batch_shape for pwc in pwc_list])
+        check_argument(
+            batch_shape is not None,
+            "All the Pwcs must have broadcastable batch shapes.",
+            {"pwc_list": pwc_list},
+        )
+
+        return Pwc(
+            _operation,
+            value_shape=value_shape,
+            durations=np.concatenate([pwc.durations for pwc in pwc_list]),
+            batch_shape=batch_shape,
+        )
+
+
 class TimeEvolutionOperatorsPwc(Node):
     """
     Calculates the unitary time-evolution operators for a system defined by a piecewise-constant
@@ -830,7 +1030,7 @@ class TimeEvolutionOperatorsPwc(Node):
     Returns
     -------
     Tensor
-        Tensor of shape [..., N, D, D], representing the unitary time evolution.
+        Tensor of shape ``(..., N, D, D)``, representing the unitary time evolution.
         The n-th element (along the -3 dimension) represents the unitary (or batch of unitaries)
         from t = 0 to ``sample_times[n]``.
 
@@ -838,6 +1038,27 @@ class TimeEvolutionOperatorsPwc(Node):
     --------
     state_evolution_pwc : Evolve a quantum state.
     time_evolution_operators_stf : Corresponding operation for `Stf` Hamiltonians.
+
+    Examples
+    --------
+    Simulate the dynamics of a single qubit, where a constant drive rotates the
+    qubit along the x-axis.
+
+    >>> initial_state = np.array([1, 0])
+    >>> sigma_x = np.array([[0, 1], [1, 0]])
+    >>> duration = np.pi
+    >>> hamiltonian = graph.constant_pwc_operator(duration=duration, operator=sigma_x / 2)
+    >>> graph.time_evolution_operators_pwc(
+    ...     hamiltonian=hamiltonian, sample_times=np.array([duration]), name="unitaries"
+    ... )
+    <Tensor: name="unitaries", operation_name="time_evolution_operators_pwc", shape=(1, 2, 2)>
+    >>> result = qctrl.functions.calculate_graph(graph=graph, output_node_names=["unitaries"])
+    >>> result.output["unitaries"]["value"].dot(initial_state)
+    array([[5.0532155e-16+0.j, 0.0000000e+00-1.j]])
+
+    See more examples in the `How to simulate quantum dynamics for noiseless systems using graphs
+    <https://docs.q-ctrl.com/boulder-opal/user-guides/how-to-simulate-quantum-dynamics-for-
+    noiseless-systems-using-graphs>`_ user guide.
     """
 
     name = "time_evolution_operators_pwc"
@@ -846,6 +1067,7 @@ class TimeEvolutionOperatorsPwc(Node):
         forge.arg("sample_times", type=np.ndarray),
     ]
     rtype = Tensor
+    categories = [Category.TIME_EVOLUTION]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -857,8 +1079,7 @@ class TimeEvolutionOperatorsPwc(Node):
             {"hamiltonian": hamiltonian},
         )
         batch_shape, value_shape = validate_batch_and_value_shapes(
-            hamiltonian,
-            "hamiltonian",
+            hamiltonian, "hamiltonian"
         )
         check_sample_times(sample_times, "sample_times")
         validate_hamiltonian(hamiltonian, "hamiltonian")
@@ -895,11 +1116,9 @@ class SamplePwc(Node):
     """
 
     name = "sample_pwc"
-    args = [
-        forge.arg("pwc", type=Pwc),
-        forge.arg("sample_times", type=np.ndarray),
-    ]
+    args = [forge.arg("pwc", type=Pwc), forge.arg("sample_times", type=np.ndarray)]
     rtype = Tensor
+    categories = [Category.BUILDING_PIECEWISE_CONSTANT_HAMILTONIANS]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
