@@ -1,16 +1,18 @@
-# Copyright 2021 Q-CTRL. All rights reserved.
+# Copyright 2022 Q-CTRL. All rights reserved.
 #
 # Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
 # copying or use of this file, via any medium, is strictly prohibited.
 # Proprietary and confidential. You may not use this file except in compliance
 # with the License. You may obtain a copy of the License at
 #
-#      https://q-ctrl.com/terms
+#    https://q-ctrl.com/terms
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS. See the
 # License for the specific language.
-"""Module for sparse."""
+"""
+Module for nodes using sparse operators.
+"""
 from typing import (
     List,
     Optional,
@@ -19,15 +21,17 @@ from typing import (
 
 import forge
 import numpy as np
-from scipy.sparse import coo_matrix
+from scipy.sparse import spmatrix
 
 from qctrlcommons.node.base import Node
+from qctrlcommons.node.documentation import Category
 from qctrlcommons.node.node_data import (
     Pwc,
     SparsePwc,
     Tensor,
 )
 from qctrlcommons.node.utils import (
+    TensorLike,
     validate_hamiltonian,
     validate_shape,
 )
@@ -53,7 +57,7 @@ class SparsePwcOperator(Node):
     ----------
     signal : Pwc
         The scalar-valued piecewise-constant function of time :math:`a(t)`.
-    operator : numpy.ndarray or scipy.sparse.coo_matrix
+    operator : numpy.ndarray or scipy.sparse.spmatrix
         The sparse operator :math:`A` to be scaled over time. If you pass a NumPy array
         then it will be internally converted into a sparse array.
 
@@ -67,17 +71,33 @@ class SparsePwcOperator(Node):
     constant_sparse_pwc_operator : Create constant `SparsePwc`\s.
     density_matrix_evolution_pwc : Evolve a quantum state in an open system.
     pwc_operator : Corresponding operation for `Pwc`\s.
+    sparse_pwc_hermitian_part : Create the Hermitian part of a `SparsePwc` operator.
     sparse_pwc_sum : Sum multiple `SparsePwc`\s.
     state_evolution_pwc : Evolve a quantum state.
+
+    Examples
+    --------
+    Create a sparse PWC operator.
+
+    >>> from scipy.sparse import coo_matrix
+    >>> sigma_x = np.array([[0, 1], [1, 0]])
+    >>> signal = graph.pwc_signal(values=np.array([1, 2, 3]), duration=0.1)
+    >>> graph.sparse_pwc_operator(signal=signal, operator=coo_matrix(sigma_x))
+    <SparsePwc: operation_name="sparse_pwc_operator", value_shape=(2, 2)>
+
+    See more examples in the `How to simulate large open system dynamics
+    <https://docs.q-ctrl.com/boulder-opal/user-guides/how-to-simulate-large-open-system-dynamics>`_
+    user guide.
     """
 
     name = "sparse_pwc_operator"
     args = [
         forge.arg("signal", type=Pwc),
-        forge.arg("operator", type=Union[np.ndarray, coo_matrix]),
+        forge.arg("operator", type=Union[np.ndarray, spmatrix]),
     ]
     kwargs = {}  # SparsePwc doesn't accept `name` as an argument.
     rtype = SparsePwc
+    categories = [Category.LARGE_SYSTEMS]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -87,21 +107,21 @@ class SparsePwcOperator(Node):
             signal.value_shape == (),
             "The signal must be scalar-valued.",
             {"signal": signal},
-            {"signal.value_shape": signal.value_shape},
+            extras={"signal.value_shape": signal.value_shape},
         )
         batch_shape = signal.batch_shape
         check_argument(
             batch_shape == (),
             "Batching is not supported for sparse_pwc.",
             {"signal": signal},
-            {"signal.batch_shape": batch_shape},
+            extras={"signal.batch_shape": batch_shape},
         )
         operator_shape = validate_shape(operator, "operator")
         check_argument(
             len(operator_shape) == 2 and operator_shape[0] == operator_shape[1],
             "Operator must be a 2D square matrix.",
             {"operator": operator},
-            {"operator.shape": operator_shape},
+            extras={"operator.shape": operator_shape},
         )
         return SparsePwc(_operation, value_shape=operator_shape)
 
@@ -114,7 +134,7 @@ class ConstantSparsePwcOperator(Node):
     ----------
     duration : float
         The duration :math:`\tau` for the resulting piecewise-constant operator.
-    operator : numpy.ndarray or scipy.sparse.coo_matrix
+    operator : numpy.ndarray or scipy.sparse.spmatrix
         The sparse operator :math:`A`. If you pass a NumPy array then it will be
         internally converted into a sparse array.
 
@@ -126,17 +146,32 @@ class ConstantSparsePwcOperator(Node):
     See Also
     --------
     constant_pwc_operator : Corresponding operation for `Pwc`\s.
+    sparse_pwc_hermitian_part : Create the Hermitian part of a `SparsePwc` operator.
     sparse_pwc_operator : Create `SparsePwc` operators.
     sparse_pwc_sum : Sum multiple `SparsePwc`\s.
+
+    Examples
+    --------
+    Create a constant sparse PWC operator.
+
+    >>> from scipy.sparse import coo_matrix
+    >>> sigma_x = np.array([[0, 1], [1, 0]])
+    >>> graph.constant_sparse_pwc_operator(duration=0.1, operator=coo_matrix(sigma_x))
+    <SparsePwc: operation_name="constant_sparse_pwc_operator", value_shape=(2, 2)>
+
+    See more examples in the `How to simulate large open system dynamics
+    <https://docs.q-ctrl.com/boulder-opal/user-guides/how-to-simulate-large-open-system-dynamics>`_
+    user guide.
     """
 
     name = "constant_sparse_pwc_operator"
     args = [
         forge.arg("duration", type=float),
-        forge.arg("operator", type=Union[np.ndarray, coo_matrix]),
+        forge.arg("operator", type=Union[np.ndarray, spmatrix]),
     ]
     kwargs = {}  # SparsePwc doesn't accept `name` as an argument.
     rtype = SparsePwc
+    categories = [Category.LARGE_SYSTEMS]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -148,7 +183,7 @@ class ConstantSparsePwcOperator(Node):
             len(operator_shape) == 2 and operator_shape[0] == operator_shape[1],
             "Operator must be a 2D square matrix.",
             {"operator": operator},
-            {"operator.shape": operator_shape},
+            extras={"operator.shape": operator_shape},
         )
         return SparsePwc(_operation, value_shape=operator_shape)
 
@@ -176,16 +211,41 @@ class SparsePwcSum(Node):
     constant_sparse_pwc_operator : Create constant `SparsePwc`\s.
     pwc_sum : Corresponding operation for `Pwc`\s.
     sparse_pwc_operator : Create `SparsePwc` operators.
+
+    Examples
+    --------
+    Sum two sparse PWC operators.
+
+    >>> from scipy.sparse import coo_matrix
+    >>> sigma_x = np.array([[0, 1], [1, 0]])
+    >>> sigma_y = np.array([[0, -1j], [1j, 0]])
+    >>> sp_x = graph.constant_sparse_pwc_operator(duration=0.1, operator=coo_matrix(sigma_x))
+    >>> sp_y = graph.constant_sparse_pwc_operator(duration=0.1, operator=coo_matrix(sigma_y))
+    >>> graph.sparse_pwc_sum([sp_x, sp_y])
+    <SparsePwc: operation_name="sparse_pwc_sum", value_shape=(2, 2)>
+
+    See more examples in the `How to simulate large open system dynamics
+    <https://docs.q-ctrl.com/boulder-opal/user-guides/how-to-simulate-large-open-system-dynamics>`_
+    user guide.
     """
 
     name = "sparse_pwc_sum"
     args = [forge.arg("terms", type=List[SparsePwc])]
     kwargs = {}  # SparsePwcSum doesn't accept `name` as an argument.
     rtype = SparsePwc
+    categories = [Category.LARGE_SYSTEMS]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
         terms = kwargs.get("terms")
+
+        check_argument(
+            isinstance(terms, list),
+            "The terms must be provided as a list.",
+            {"terms": terms},
+            extras={"type(terms)": type(terms)},
+        )
+
         shape_0 = terms[0].value_shape
         for i in range(1, len(terms)):
             shape = terms[i].value_shape
@@ -193,7 +253,7 @@ class SparsePwcSum(Node):
                 shape == shape_0,
                 "All the terms must have the same shape.",
                 {"terms": terms},
-                {
+                extras={
                     f"terms[{0}].value_shape": shape_0,
                     f"terms[{i}].value_shape": shape,
                 },
@@ -201,6 +261,53 @@ class SparsePwcSum(Node):
         return SparsePwc(_operation, value_shape=shape_0)
 
 
+class SparsePwcHermitianPart(Node):
+    r"""
+    Creates the Hermitian part of a piecewise-constant operator.
+
+    Parameters
+    ----------
+    operator : SparsePwc
+        The operator :math:`A(t)`.
+
+    Returns
+    -------
+    SparsePwc
+        The Hermitian part :math:`\frac{1}{2}(A(t)+A^\dagger(t))`.
+
+    See Also
+    --------
+    hermitian_part : Create the Hermitian part of an operator.
+    sparse_pwc_operator : Create `SparsePwc`\s.
+
+    Examples
+    --------
+    Create a Hermitian sparse PWC operator.
+
+    >>> from scipy.sparse import coo_matrix
+    >>> sigma_m = np.array([[0, 1], [0, 0]])
+    >>> sp_m = graph.constant_sparse_pwc_operator(duration=0.1, operator=coo_matrix(sigma_m))
+    >>> graph.sparse_pwc_hermitian_part(sp_m)
+    <SparsePwc: operation_name="sparse_pwc_hermitian_part", value_shape=(2, 2)>
+    """
+
+    name = "sparse_pwc_hermitian_part"
+    args = [forge.arg("operator", type=SparsePwc)]
+    kwargs = {}  # SparsePwc is not fetchable.
+    rtype = SparsePwc
+    categories = [Category.LARGE_SYSTEMS]
+
+    @classmethod
+    def create_node_data(cls, _operation, **kwargs):
+        operator = kwargs.get("operator")
+        check_argument(
+            isinstance(operator, SparsePwc),
+            "The operator must be a SparsePwc.",
+            {"operator": operator},
+        )
+        return SparsePwc(_operation, value_shape=operator.value_shape)
+
+
 class StateEvolutionPwc(Node):
     r"""
     Calculates the time evolution of a state generated by a piecewise-constant
@@ -209,14 +316,14 @@ class StateEvolutionPwc(Node):
     Parameters
     ----------
     initial_state : Tensor or np.ndarray
-        The initial state as a Tensor or np.ndarray of shape ``[D]``.
+        The initial state as a Tensor or np.ndarray of shape ``(D,)``.
     hamiltonian : Pwc or SparsePwc
         The control Hamiltonian. Uses sparse matrix multiplication if of type
         `SparsePwc`, which can be more efficient for large operators that are
         relatively sparse (contain mostly zeros).
     krylov_subspace_dimension : Tensor or int
         The dimension of the Krylov subspace `k` for the Lanczos method.
-    sample_times :  np.ndarray(1D, real), optional
+    sample_times : np.ndarray(1D, real), optional
         The N times at which you want to sample the state. Elements must be non-negative
         and strictly increasing, with a supremum that is the duration of the `hamiltonian`.
         If omitted only the evolved state at the final time of the control Hamiltonian
@@ -227,7 +334,7 @@ class StateEvolutionPwc(Node):
     Returns
     -------
     Tensor
-        Tensor of shape ``[N, D]`` or ``[D]`` if `sample_times` is omitted
+        Tensor of shape ``(N, D)`` or ``(D,)`` if `sample_times` is omitted
         representing the state evolution. The n-th element (along the first
         dimension) represents the state at ``sample_times[n]`` evolved from
         the initial state.
@@ -244,7 +351,8 @@ class StateEvolutionPwc(Node):
     for an example of calculating state evolution with `time_evolution_operators_pwc`.
 
     .. _How to simulate quantum dynamics for noiseless systems using graphs:
-        https://docs.q-ctrl.com/boulder-opal/user-guides/how-to-simulate-quantum-dynamics-for-noiseless-systems-using-graphs
+        https://docs.q-ctrl.com/boulder-opal/user-guides/how-to-simulate-quantum-
+        dynamics-for-noiseless-systems-using-graphs
 
     See Also
     --------
@@ -274,20 +382,34 @@ class StateEvolutionPwc(Node):
     integration. However, increasing the Krylov subspace dimension or subdividing the Hamiltonian
     into shorter piecewise-constant segments can reduce the integration error, at the expense of
     longer computation times.
+
+    Examples
+    --------
+    See example in the `How to optimize controls on large sparse Hamiltonians
+    <https://docs.q-ctrl.com/boulder-opal/user-guides/how-to-optimize-controls-on-
+    large-sparse-hamiltonians>`_ user guide.
     """
 
     name = "state_evolution_pwc"
     args = [
-        forge.arg("initial_state", type=Union[Tensor, np.ndarray]),
+        forge.arg("initial_state", type=TensorLike),
         forge.arg("hamiltonian", type=Union[Pwc, SparsePwc]),
         forge.arg("krylov_subspace_dimension", type=Union[int, Tensor]),
         forge.arg("sample_times", type=Optional[np.ndarray], default=None),
     ]
     rtype = Tensor
+    categories = [Category.LARGE_SYSTEMS, Category.TIME_EVOLUTION]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
         initial_state = kwargs.get("initial_state")
+        state_shape = validate_shape(initial_state, "initial_state")
+        check_argument(
+            len(state_shape) == 1,
+            "The initial state must be 1D.",
+            {"initial_state": initial_state},
+            extras={"initial_state.shape": state_shape},
+        )
         hamiltonian = kwargs.get("hamiltonian")
         sample_times = kwargs.get("sample_times")
         check_argument_numeric(initial_state, "initial_state")
@@ -298,22 +420,28 @@ class StateEvolutionPwc(Node):
             "The Hamiltonian must be a SparsePwc or a Pwc.",
             {"hamiltonian": hamiltonian},
         )
-        value_shape_state = validate_shape(initial_state, "initial_state")
-        value_shape_hamiltonian = hamiltonian.value_shape
+        if isinstance(hamiltonian, Pwc):
+            check_argument(
+                hamiltonian.batch_shape == (),
+                "Hamiltonian cannot contain a batch.",
+                {"hamiltonian": hamiltonian},
+                extras={"hamiltonian.batch_shape": hamiltonian.batch_shape},
+            )
+        hamiltonian_value_shape = hamiltonian.value_shape
         validate_hamiltonian(hamiltonian, "hamiltonian")
         check_argument(
-            value_shape_state[0] == value_shape_hamiltonian[-1],
+            state_shape[0] == hamiltonian_value_shape[-1],
             "The Hilbert space dimensions of the state and the Hamiltonian must be equal.",
             {"initial_state": initial_state, "hamiltonian": hamiltonian},
             extras={
-                "initial_state.shape": value_shape_state,
-                "hamiltonian.value_shape": value_shape_hamiltonian,
+                "initial_state.shape": state_shape,
+                "hamiltonian.value_shape": hamiltonian_value_shape,
             },
         )
         if sample_times is None:
-            shape = value_shape_state
+            shape = state_shape
         else:
-            shape = (len(sample_times),) + value_shape_state
+            shape = (len(sample_times),) + state_shape
         return Tensor(_operation, shape=shape)
 
 
@@ -375,6 +503,24 @@ class EstimatedKrylovSubspaceDimensionLanczos(Node):
     .. [2] `M. Hochbruck and C. Lubich,
            SIAM Journal on Numerical Analysis 34, 1911 (1997).
            <https://doi.org/10.1137/S0036142995280572>`_
+
+    Examples
+    --------
+    >>> graph.estimated_krylov_subspace_dimension_lanczos(
+    ...     spectral_range=30.0,
+    ...     duration=5e-7,
+    ...     maximum_segment_duration=2.5e-8,
+    ...     error_tolerance=1e-5,
+    ...     name="dim",
+    ... )
+    <Tensor: name="dim", operation_name="estimated_krylov_subspace_dimension_lanczos", shape=()>
+    >>> result = qctrl.functions.calculate_graph(graph=graph, output_node_names=["dim"])
+    >>> result.output["dim"]["value"]
+    2
+
+    See more examples in the `How to optimize controls on large sparse Hamiltonians
+    <https://docs.q-ctrl.com/boulder-opal/user-guides/how-to-optimize-controls-on-
+    large-sparse-hamiltonians>`_ user guide.
     """
 
     name = "estimated_krylov_subspace_dimension_lanczos"
@@ -385,6 +531,7 @@ class EstimatedKrylovSubspaceDimensionLanczos(Node):
         forge.arg("error_tolerance", type=float, default=1e-6),
     ]
     rtype = Tensor
+    categories = [Category.LARGE_SYSTEMS, Category.TIME_EVOLUTION]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -426,7 +573,7 @@ class SpectralRange(Node):
 
     Parameters
     ----------
-    operator : np.ndarray or scipy.sparse.coo_matrix or Tensor
+    operator : np.ndarray or scipy.sparse.spmatrix or Tensor
         The Hermitian operator :math:`M` whose range of eigenvalues you
         want to determine.
     iteration_count : int, optional
@@ -435,7 +582,7 @@ class SpectralRange(Node):
         number to make the estimation run faster.
     seed : int, optional
         The random seed that the function uses to choose the initial random
-        vector :math:`\left| r \right\rangle`. Defaults to ``None``, which
+        vector :math:`\left| r \right\rangle`. Defaults to None, which
         means that the function uses a different seed in each run.
     name : str, optional
         The name of the node.
@@ -475,15 +622,31 @@ class SpectralRange(Node):
 
     The same procedure applied to the matrix :math:`M-V` allows the function
     to find the eigenvalue at the opposite end of the spectral range.
+
+    Examples
+    --------
+    >>> operator = np.diag([10, 40])
+    >>> graph.spectral_range(operator, name="spectral_range")
+    <Tensor: name="spectral_range", operation_name="spectral_range", shape=()>
+    >>> result = qctrl.functions.calculate_graph(
+    ...     graph=graph, output_node_names=["spectral_range"]
+    ... )
+    >>> result.output["spectral_range"]["value"]
+    30.0
+
+    See more examples in the `How to optimize controls on large sparse Hamiltonians
+    <https://docs.q-ctrl.com/boulder-opal/user-guides/how-to-optimize-controls-on
+    -large-sparse-hamiltonians>`_ user guide.
     """
 
     name = "spectral_range"
     args = [
-        forge.arg("operator", type=Union[Tensor, np.ndarray, coo_matrix]),
+        forge.arg("operator", type=Union[Tensor, np.ndarray, spmatrix]),
         forge.arg("iteration_count", type=int, default=3000),
         forge.arg("seed", type=Optional[int], default=None),
     ]
     rtype = Tensor
+    categories = [Category.LARGE_SYSTEMS, Category.TIME_EVOLUTION]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
