@@ -1,11 +1,11 @@
-# Copyright 2021 Q-CTRL. All rights reserved.
+# Copyright 2022 Q-CTRL. All rights reserved.
 #
 # Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
 # copying or use of this file, via any medium, is strictly prohibited.
 # Proprietary and confidential. You may not use this file except in compliance
 # with the License. You may obtain a copy of the License at
 #
-#      https://q-ctrl.com/terms
+#    https://q-ctrl.com/terms
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS. See the
@@ -25,8 +25,9 @@ import numpy as np
 from qctrlcommons.node.base import Node
 from qctrlcommons.node.deprecation import (
     deprecated_node,
-    deprecated_parameter,
+    rename_parameter,
 )
+from qctrlcommons.node.documentation import Category
 from qctrlcommons.node.node_data import (
     ConvolutionKernel,
     Pwc,
@@ -34,17 +35,16 @@ from qctrlcommons.node.node_data import (
     Tensor,
 )
 from qctrlcommons.node.utils import (
+    TensorLike,
     validate_batch_and_value_shapes,
     validate_hamiltonian,
-    validate_inputs_real_fourier_signal,
     validate_shape,
 )
 from qctrlcommons.preconditions import (
     check_argument,
     check_argument_integer,
-    check_argument_iteratable,
+    check_argument_iterable,
     check_argument_numeric,
-    check_duration,
     check_operator,
     check_sample_times,
 )
@@ -76,21 +76,17 @@ class StfOperator(Node):
     """
 
     name = "stf_operator"
-    args = [
-        forge.arg("signal", type=Stf),
-        forge.arg("operator", type=Union[np.ndarray, Tensor]),
-    ]
+    args = [forge.arg("signal", type=Stf), forge.arg("operator", type=TensorLike)]
     kwargs = {}  # Stfs don't accept name as an argument.
     rtype = Stf
+    categories = [Category.BUILDING_SMOOTH_HAMILTONIANS]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
         signal = kwargs.get("signal")
         operator = kwargs.get("operator")
         check_argument(
-            isinstance(signal, Stf),
-            "The signal must be an Stf.",
-            {"signal": signal},
+            isinstance(signal, Stf), "The signal must be an Stf.", {"signal": signal}
         )
         batch_shape, _ = validate_batch_and_value_shapes(signal, "signal")
         value_shape = validate_shape(operator, "operator")
@@ -101,11 +97,7 @@ class StfOperator(Node):
             {"operator": operator},
             extras={"operator.shape": value_shape},
         )
-        return Stf(
-            _operation,
-            value_shape=value_shape,
-            batch_shape=batch_shape,
-        )
+        return Stf(_operation, value_shape=value_shape, batch_shape=batch_shape)
 
 
 class ConstantStfOperator(Node):
@@ -127,25 +119,22 @@ class ConstantStfOperator(Node):
     See Also
     --------
     constant_pwc_operator : Corresponding operation for `Pwc`\s.
-    constant_stf: Create a batch of constant `Stf`\s.
+    constant_stf : Create a batch of constant `Stf`\s.
     stf_operator : Create an `Stf` operator.
     """
 
     name = "constant_stf_operator"
-    args = [forge.arg("operator", type=Union[np.ndarray, Tensor])]
+    args = [forge.arg("operator", type=TensorLike)]
     kwargs = {}  # Stfs don't accept name as an argument.
     rtype = Stf
+    categories = [Category.BUILDING_SMOOTH_HAMILTONIANS]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
         operator = kwargs.get("operator")
         shape = validate_shape(operator, "operator")
         check_operator(operator, "operator")
-        return Stf(
-            _operation,
-            value_shape=shape[-2:],
-            batch_shape=shape[:-2],
-        )
+        return Stf(_operation, value_shape=shape[-2:], batch_shape=shape[:-2])
 
 
 class ConstantStf(Node):
@@ -185,6 +174,7 @@ class ConstantStf(Node):
     ]
     kwargs = {}  # Stfs don't accept name as an argument.
     rtype = Stf
+    categories = [Category.BUILDING_SMOOTH_HAMILTONIANS]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -200,7 +190,7 @@ class ConstantStf(Node):
             "The number of batch dimensions must not be larger than the number "
             "of dimensions of the input constant.",
             {"constant": constant, "batch_dimension_count": batch_dimension_count},
-            {"Number of value dimensions": len(shape)},
+            extras={"Number of value dimensions": len(shape)},
         )
         check_argument(
             batch_dimension_count >= 0,
@@ -232,30 +222,27 @@ class StfSum(Node):
 
     See Also
     --------
+    hermitian_part : Hermitian part of an `Stf` operator.
     pwc_sum : Corresponding operation for `Pwc`\s.
     stf_operator : Create an `Stf` operator.
-    stf_operator_hermitian_part : Hermitian part of an `Stf` operator.
-    stf_sum : Sum multiple `Stf`\s.
     """
 
     name = "stf_sum"
     args = [forge.arg("terms", type=List[Stf])]
     kwargs = {}  # Stfs don't accept name as an argument.
     rtype = Stf
+    categories = [Category.BUILDING_SMOOTH_HAMILTONIANS]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
         terms = kwargs.get("terms")
-        check_argument_iteratable(terms, "terms")
+        check_argument_iterable(terms, "terms")
         check_argument(
             all(isinstance(term, Stf) for term in terms),
             "Each of the terms must be an Stf.",
             {"terms": terms},
         )
-        batch_shape, value_shape = validate_batch_and_value_shapes(
-            terms[0],
-            "terms[0]",
-        )
+        batch_shape, value_shape = validate_batch_and_value_shapes(terms[0], "terms[0]")
         check_argument(
             all(
                 (
@@ -267,13 +254,10 @@ class StfSum(Node):
             "All the terms must have the same shape.",
             {"terms": terms},
         )
-        return Stf(
-            _operation,
-            value_shape=value_shape,
-            batch_shape=batch_shape,
-        )
+        return Stf(_operation, value_shape=value_shape, batch_shape=batch_shape)
 
 
+@deprecated_node(updated_node_name="hermitian_part")  # deprecated 2022/04/21
 class StfOperatorHermitianPart(Node):
     r"""
     Creates the Hermitian part of an operator-valued sampleable function.
@@ -292,12 +276,24 @@ class StfOperatorHermitianPart(Node):
     --------
     pwc_operator_hermitian_part : Corresponding operation for `Pwc`\s.
     stf_operator : Create an `Stf` operator.
+
+    Examples
+    --------
+    >>> operator = stf_signal * np.array([[0, 1, 0], [0, 0, np.sqrt(2)], [0, 0, 0]])
+    >>> operator
+    <Stf: operation_name="multiply", value_shape=(3, 3), batch_shape=()>
+    >>> hamiltonian = graph.stf_operator_hermitian_part(operator)
+
+    Refer to the `How to perform model-based optimization with user-defined basis functions
+    <https://docs.q-ctrl.com/boulder-opal/user-guides/how-to-perform-model-based-optimization-with-user-defined-basis-functions>`_
+    user guide to find the example in context.
     """
 
     name = "stf_operator_hermitian_part"
     args = [forge.arg("operator", type=Stf)]
     kwargs = {}  # Stfs don't accept name as an argument.
     rtype = Stf
+    categories = [Category.BUILDING_SMOOTH_HAMILTONIANS]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -308,11 +304,7 @@ class StfOperatorHermitianPart(Node):
             {"operator": operator},
         )
         batch_shape, value_shape = validate_batch_and_value_shapes(operator, "operator")
-        return Stf(
-            _operation,
-            value_shape=value_shape,
-            batch_shape=batch_shape,
-        )
+        return Stf(_operation, value_shape=value_shape, batch_shape=batch_shape)
 
 
 class DiscretizeStf(Node):
@@ -351,6 +343,14 @@ class DiscretizeStf(Node):
         the sampleable function (or batch of piecewise-constant functions, if
         you provided a batch of sampleable functions).
 
+    See Also
+    --------
+    convolve_pwc : Create an `Stf` by convolving a `Pwc` with a kernel.
+    identity_stf : Create an `Stf` representing the identity function.
+    sample_stf : Sample an `Stf` at given times.
+    :func:`.utils.filter_and_resample_pwc` :
+        Filter a `Pwc` with a sinc filter and resample it.
+
     Notes
     -----
     The resulting function :math:`w(t)` is piecewise-constant with :math:`N`
@@ -365,11 +365,21 @@ class DiscretizeStf(Node):
         w_n = \frac{1}{M}
         \sum_{m=1}^M v\left(t_{n-1} + \left(m-\tfrac{1}{2}\right) \frac{\tau}{MN} \right).
 
-    See Also
+    Examples
     --------
-    convolve_pwc : Create an `Stf` by convolving a `Pwc` with a kernel.
-    identity_stf : Create an `Stf` representing the identity function.
-    sample_stf : Sample an `Stf` at given times.
+    Create discretized Gaussian signal.
+
+    >>> times = graph.identity_stf()
+    >>> gaussian_signal = graph.exp(- (times - 5e-6) ** 2 / 2e-6 ** 2) / 2e-6
+    >>> discretized_gamma_signal = graph.discretize_stf(
+    ...     stf=gaussian_signal, duration=10e-6, segment_count=256, name="gamma"
+    ... )
+    >>> discretized_gamma_signal
+    <Pwc: name="gamma", operation_name="discretize_stf", value_shape=(), batch_shape=()>
+
+    Refer to the `How to create dephasing and amplitude robust single-qubit gates
+    <https://docs.q-ctrl.com/boulder-opal/user-guides/how-to-create-dephasing-and-amplitude-robust-single-qubit-gates>`_
+    user guide to find the example in context.
     """
 
     name = "discretize_stf"
@@ -380,16 +390,14 @@ class DiscretizeStf(Node):
         forge.arg("sample_count_per_segment", type=int, default=1),
     ]
     rtype = Pwc
+    categories = [Category.FILTERING_AND_DISCRETIZING]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
         stf = kwargs.get("stf")
         duration = kwargs.get("duration")
         segment_count = kwargs.get("segment_count")
-        batch_shape, value_shape = validate_batch_and_value_shapes(
-            stf,
-            "stf",
-        )
+        batch_shape, value_shape = validate_batch_and_value_shapes(stf, "stf")
         check_argument_integer(segment_count, "segment_count")
         check_argument(
             segment_count > 0,
@@ -435,13 +443,35 @@ class TimeEvolutionOperatorsStf(Node):
     Returns
     -------
     Tensor
-        Tensor of shape [..., N, D, D], representing the unitary time evolution.
+        Tensor of shape ``(..., N, D, D)``, representing the unitary time evolution.
         The n-th element (along the -3 dimension) represents the unitary (or batch of unitaries)
         from t = 0 to ``sample_times[n]``.
 
     See Also
     --------
     time_evolution_operators_pwc : Corresponding operation for `Pwc` Hamiltonians.
+
+    Examples
+    --------
+    Simulate the dynamics of a qubit, where a simple Gaussian drive rotate the qubit
+    along the x-axis.
+
+    >>> duration = np.pi
+    >>> initial_state = np.array([1, 0])
+    >>> sigma_x = np.array([[0, 1], [1, 0]])
+    >>> time = graph.identity_stf()
+    >>> gaussian_drive = graph.exp(-(time ** 2))
+    >>> hamiltonian = gaussian_drive * sigma_x * np.sqrt(np.pi) / 2
+    >>> graph.time_evolution_operators_stf(
+    ...     hamiltonian=hamiltonian,
+    ...     sample_times=np.array([duration]),
+    ...     evolution_times=np.linspace(0, duration, 200),
+    ...     name="unitaries",
+    ... )
+    <Tensor: name="unitaries", operation_name="time_evolution_operators_stf", shape=(1, 2, 2)>
+    >>> result = qctrl.functions.calculate_graph(graph=graph, output_node_names=["unitaries"])
+    >>> result.output["unitaries"]["value"].dot(initial_state)
+    array([[0.70711169 + 0.0j, 0.0 - 0.70710187j]])
     """
 
     name = "time_evolution_operators_stf"
@@ -451,6 +481,7 @@ class TimeEvolutionOperatorsStf(Node):
         forge.arg("evolution_times", type=Optional[np.ndarray], default=None),
     ]
     rtype = Tensor
+    categories = [Category.TIME_EVOLUTION]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -463,8 +494,7 @@ class TimeEvolutionOperatorsStf(Node):
             {"hamiltonian": hamiltonian},
         )
         batch_shape, value_shape = validate_batch_and_value_shapes(
-            hamiltonian,
-            "hamiltonian",
+            hamiltonian, "hamiltonian"
         )
         check_sample_times(sample_times, "sample_times")
         validate_hamiltonian(hamiltonian, "hamiltonian")
@@ -487,9 +517,6 @@ class TimeEvolutionOperatorsStf(Node):
         return Tensor(_operation, shape=shape)
 
 
-@deprecated_parameter(  # deprecated 2021/05/24
-    deprecated_name="kernel_integral", new_name="kernel"
-)
 class ConvolvePwc(Node):
     r"""
     Creates the convolution of a piecewise-constant function with a kernel.
@@ -529,36 +556,52 @@ class ConvolvePwc(Node):
     pwc : Create piecewise-constant functions.
     sample_stf : Sample an `Stf` at given times.
     sinc_convolution_kernel : Create a convolution kernel representing the sinc function.
+    :func:`.utils.filter_and_resample_pwc` :
+        Filter a `Pwc` with a sinc filter and resample it.
+
+    Examples
+    --------
+    Filter a piecewise-constant signal using a Gaussian convolution kernel.
+
+    >>> gaussian_kernel = graph.gaussian_convolution_kernel(std=1.0, offset=3.0)
+    >>> gaussian_kernel
+    <ConvolutionKernel: operation_name="gaussian_convolution_kernel">
+    >>> pwc_signal
+    <Pwc: name="alpha", operation_name="pwc_signal", value_shape=(), batch_shape=()>
+    >>> filtered_signal = graph.convolve_pwc(pwc=pwc_signal, kernel=gaussian_kernel)
+    >>> filtered_signal
+    <Stf: operation_name="convolve_pwc", value_shape=(), batch_shape=()>
+
+    Refer to the `How to add smoothing and band-limits to optimized controls
+    <https://docs.q-ctrl.com/boulder-opal/user-guides/how-to-add-smoothing-and-band-limits-to-optimized-controls>`_
+    user guide to find the example in context.
     """
 
     name = "convolve_pwc"
-    args = [
-        forge.arg("pwc", type=Pwc),
-        forge.arg("kernel", type=ConvolutionKernel),
-    ]
+    args = [forge.arg("pwc", type=Pwc), forge.arg("kernel", type=ConvolutionKernel)]
     kwargs = {}  # Stfs don't accept name as an argument.
     rtype = Stf
+    categories = [Category.FILTERING_AND_DISCRETIZING]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
         pwc = kwargs.get("pwc")
         batch_shape, value_shape = validate_batch_and_value_shapes(pwc, "pwc")
-        return Stf(
-            _operation,
-            value_shape=value_shape,
-            batch_shape=batch_shape,
-        )
+        return Stf(_operation, value_shape=value_shape, batch_shape=batch_shape)
 
 
+@rename_parameter(  # deprecated 2022/04/11
+    old_name="cut_off_frequency", new_name="cutoff_frequency"
+)
 class SincConvolutionKernel(Node):
     r"""
     Creates a convolution kernel representing the sinc function.
 
-    Use this kernel to eliminate frequencies above a certain cut-off.
+    Use this kernel to eliminate frequencies above a certain cutoff.
 
     Parameters
     ----------
-    cut_off_frequency : float or Tensor
+    cutoff_frequency : float or Tensor
         Upper limit :math:`\omega_c` of the range of frequencies that you want
         to preserve. The filter eliminates components of the signal that have
         higher frequencies.
@@ -585,27 +628,38 @@ class SincConvolutionKernel(Node):
     --------
     convolve_pwc : Create an `Stf` by convolving a `Pwc` with a kernel.
     gaussian_convolution_kernel : Create a convolution kernel representing a normalized Gaussian.
+    :func:`.utils.filter_and_resample_pwc` :
+        Filter a `Pwc` with a sinc filter and resample it.
+
+    Examples
+    --------
+    Filter a signal by convolving it with a sinc kernel.
+
+    >>> sinc_kernel = graph.sinc_convolution_kernel(cutoff_frequency=300e6)
+    >>> sinc_kernel
+    <ConvolutionKernel: operation_name="sinc_convolution_kernel">
+    >>> pwc_signal
+    <Pwc: name="pwc_signal_#1", operation_name="pwc_signal", value_shape=(), batch_shape=()>
+    >>> filtered_signal = graph.convolve_pwc(pwc=pwc_signal, kernel=sinc_kernel)
+    >>> filtered_signal
+    <Stf: operation_name="convolve_pwc", value_shape=(), batch_shape=()>
+
+    Refer to the `How to create leakage-robust single-qubit gates
+    <https://docs.q-ctrl.com/boulder-opal/user-guides/how-to-create-leakage-robust-single-qubit-gates>`_
+    user guide to find the example in context.
     """
 
     name = "sinc_convolution_kernel"
-    args = [
-        forge.arg("cut_off_frequency", type=Union[float, Tensor]),
-    ]
+    args = [forge.arg("cutoff_frequency", type=Union[float, Tensor])]
     kwargs = {}  # ConvolutionKernels don't accept name as an argument.
     rtype = ConvolutionKernel
+    categories = [Category.FILTERING_AND_DISCRETIZING]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
         return ConvolutionKernel(_operation)
 
 
-@deprecated_node("sinc_convolution_kernel")  # deprecated 2021/05/24
-class SincIntegralFunction(SincConvolutionKernel):
-    """Deprecated node."""
-
-    name = "sinc_integral_function"
-
-
 class GaussianConvolutionKernel(Node):
     r"""
     Creates a convolution kernel representing a normalized Gaussian.
@@ -643,6 +697,23 @@ class GaussianConvolutionKernel(Node):
     --------
     convolve_pwc : Create an `Stf` by convolving a `Pwc` with a kernel.
     sinc_convolution_kernel : Create a convolution kernel representing the sinc function.
+
+    Examples
+    --------
+    Filter a signal by convolving it with a Gaussian kernel.
+
+    >>> gaussian_kernel = graph.gaussian_convolution_kernel(std=1.0, offset=3.0)
+    >>> gaussian_kernel
+    <ConvolutionKernel: operation_name="gaussian_convolution_kernel">
+    >>> signal
+    <Pwc: name="alpha", operation_name="pwc_signal", value_shape=(), batch_shape=()>
+    >>> filtered_signal = graph.convolve_pwc(pwc=signal, kernel=gaussian_kernel)
+    >>> filtered_signal
+    <Stf: operation_name="convolve_pwc", value_shape=(), batch_shape=()>
+
+    Refer to the `How to characterize the bandwidth of a transmission line using a qubit as a probe
+    <https://docs.q-ctrl.com/boulder-opal/user-guides/how-to-characterize-the-bandwidth-of-a-transmission-line-using-a-qubit-as-a-probe>`_
+    user guide to find the example in context.
     """
 
     name = "gaussian_convolution_kernel"
@@ -652,19 +723,13 @@ class GaussianConvolutionKernel(Node):
     ]
     kwargs = {}  # ConvolutionKernels don't accept name as an argument.
     rtype = ConvolutionKernel
+    categories = [Category.FILTERING_AND_DISCRETIZING]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
         return ConvolutionKernel(_operation)
 
 
-@deprecated_node("gaussian_convolution_kernel")  # deprecated 2021/05/24
-class GaussianIntegralFunction(GaussianConvolutionKernel):
-    """Deprecated node."""
-
-    name = "gaussian_integral_function"
-
-
 class SampleStf(Node):
     """
     Samples an Stf at the given times.
@@ -693,11 +758,9 @@ class SampleStf(Node):
     """
 
     name = "sample_stf"
-    args = [
-        forge.arg("stf", type=Stf),
-        forge.arg("sample_times", type=np.ndarray),
-    ]
+    args = [forge.arg("stf", type=Stf), forge.arg("sample_times", type=np.ndarray)]
     rtype = Tensor
+    categories = [Category.BUILDING_SMOOTH_HAMILTONIANS]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -713,7 +776,7 @@ class SampleStf(Node):
 
 class IdentityStf(Node):
     r"""
-    Returns an Stf representing the identity function, f(t) = t.
+    Creates an Stf representing the identity function, f(t) = t.
 
     Returns
     -------
@@ -725,129 +788,29 @@ class IdentityStf(Node):
     constant_stf: Create a batch of constant `Stf`\s.
     discretize_stf : Discretize an `Stf` into a `Pwc`.
     sample_stf : Sample an `Stf` at given times.
+
+    Examples
+    --------
+    Create Gaussian pulse.
+
+    >>> time = graph.identity_stf()
+    >>> time
+    <Stf: operation_name="identity_stf", value_shape=(), batch_shape=()>
+    >>> gaussian = graph.exp(- time ** 2)
+    >>> gaussian
+    <Stf: operation_name="exp", value_shape=(), batch_shape=()>
+
+    Refer to the `How to perform model-based optimization with user-defined basis functions
+    <https://docs.q-ctrl.com/boulder-opal/user-guides/how-to-perform-model-based-optimization-with-user-defined-basis-functions>`_
+    user guide to find the example in context.
     """
 
     name = "identity_stf"
     args = []
     kwargs = {}  # Stfs don't accept name as an argument.
     rtype = Stf
+    categories = [Category.BUILDING_SMOOTH_HAMILTONIANS]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
-        return Stf(
-            _operation,
-            value_shape=(),
-            batch_shape=(),
-        )
-
-
-class RealFourierStfSignal(Node):
-    r"""
-    Creates a real sampleable signal constructed from Fourier components.
-
-    Use this function to create a signal defined in terms of Fourier (sine/cosine)
-    basis signals that can be optimized by varying their coefficients and, optionally,
-    their frequencies.
-
-    Parameters
-    ----------
-    duration : float
-        The total duration :math:`\tau` of the signal.
-    initial_coefficient_lower_bound : float, optional
-        The lower bound :math:`c_\mathrm{min}` on the initial coefficient
-        values. Defaults to -1.
-    initial_coefficient_upper_bound : float, optional
-        The upper bound :math:`c_\mathrm{max}` on the initial coefficient
-        values. Defaults to 1.
-    fixed_frequencies : list[float], optional
-        The fixed non-zero frequencies :math:`\{f_m\}` to use for the Fourier
-        basis. Must be non-empty if provided. Must be specified in the inverse
-        units of `duration` (for example if `duration` is in seconds, these
-        values must be given in Hertz).
-    optimizable_frequency_count : int, optional
-        The number of non-zero frequencies :math:`M` to use, if the
-        frequencies can be optimized. Must be greater than zero if provided.
-    randomized_frequency_count : int, optional
-        The number of non-zero frequencies :math:`M` to use, if the
-        frequencies are to be randomized but fixed. Must be greater than zero
-        if provided.
-
-    Returns
-    -------
-    Stf(1D, real)
-        The optimizable, real-valued, sampleable signal built from the
-        appropriate Fourier components.
-
-    Warnings
-    --------
-    You must provide exactly one of `fixed_frequencies`, `optimizable_variable`,
-    or `randomized_frequency_count`.
-
-    See Also
-    --------
-    real_fourier_pwc_signal : Corresponding operation for `Pwc`.
-
-    Notes
-    -----
-    This function sets the basis signal frequencies :math:`\{f_m\}`
-    depending on the chosen mode:
-
-    * For fixed frequencies, you provide the frequencies directly.
-    * For optimizable frequencies, you provide the number of frequencies
-      :math:`M`, and this function creates :math:`M` unbounded optimizable
-      variables :math:`\{f_m\}`, with initial values in the ranges
-      :math:`\{[(m-1)/\tau, m/\tau]\}`.
-    * For randomized frequencies, you provide the number of frequencies
-      :math:`M`, and this function creates :math:`M` randomized constants
-      :math:`\{f_m\}` in the ranges :math:`\{[(m-1)/\tau, m/\tau]\}`.
-
-    After this function creates the :math:`M` frequencies :math:`\{f_m\}`, it
-    produces the signal
-
-    .. math::
-        \alpha^\prime(t) = v_0 +
-        \sum_{m=1}^M [ v_m \cos(2\pi t f_m) + w_m \sin(2\pi t f_m) ],
-
-    where :math:`\{v_m,w_m\}` are (unbounded) optimizable variables, with
-    initial values bounded by :math:`c_\mathrm{min}` and
-    :math:`c_\mathrm{max}`. This function produces the final signal :math:`\alpha(t)`.
-
-    You can use the signals created by this function for chopped random basis
-    (CRAB) optimization [1]_.
-
-    References
-    ----------
-    .. [1] `P. Doria, T. Calarco, and S. Montangero, Physical Review Letters 106, 190501 (2011).
-           <https://doi.org/10.1103/PhysRevLett.106.190501>`_
-    """
-
-    name = "real_fourier_stf_signal"
-    optimizable_variable = True
-    args = [
-        forge.arg("duration", type=float),
-        forge.arg("initial_coefficient_lower_bound", type=float, default=-1),
-        forge.arg("initial_coefficient_upper_bound", type=float, default=1),
-        forge.arg("fixed_frequencies", type=Optional[List[float]], default=None),
-        forge.arg("optimizable_frequency_count", type=Optional[int], default=None),
-        forge.arg("randomized_frequency_count", type=Optional[int], default=None),
-    ]
-    kwargs = {}  # Stfs don't accept name as an argument.
-    rtype = Stf
-
-    @classmethod
-    def create_node_data(cls, _operation, **kwargs):
-        duration = kwargs.get("duration")
-        check_duration(duration, "duration")
-
-        fixed_frequencies = kwargs.get("fixed_frequencies")
-        optimizable_frequency_count = kwargs.get("optimizable_frequency_count")
-        randomized_frequency_count = kwargs.get("randomized_frequency_count")
-        validate_inputs_real_fourier_signal(
-            fixed_frequencies, optimizable_frequency_count, randomized_frequency_count
-        )
-
-        return Stf(
-            _operation,
-            value_shape=(),
-            batch_shape=(),
-        )
+        return Stf(_operation, value_shape=(), batch_shape=())
