@@ -1,19 +1,23 @@
-# Copyright 2021 Q-CTRL. All rights reserved.
+# Copyright 2022 Q-CTRL. All rights reserved.
 #
 # Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
 # copying or use of this file, via any medium, is strictly prohibited.
 # Proprietary and confidential. You may not use this file except in compliance
 # with the License. You may obtain a copy of the License at
 #
-#      https://q-ctrl.com/terms
+#    https://q-ctrl.com/terms
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS. See the
 # License for the specific language.
-"""Utilities for commons nodes."""
+"""
+Utilities for commons nodes.
+"""
+from numbers import Number
 from typing import Union
 
 import numpy as np
+from scipy.sparse import spmatrix
 
 from qctrlcommons.exceptions import QctrlException
 from qctrlcommons.node.node_data import (
@@ -22,47 +26,62 @@ from qctrlcommons.node.node_data import (
     Tensor,
 )
 from qctrlcommons.preconditions import (
+    QctrlArgumentsValueError,
     check_argument,
     check_argument_integer,
+    check_argument_iterable,
     check_argument_numeric,
     check_argument_operator,
 )
 
+TensorLike = Union[np.ndarray, Tensor]
+TensorLikeOrFunction = Union[np.ndarray, Tensor, Pwc, Stf]
 NumericOrFunction = Union[float, complex, np.ndarray, Tensor, Pwc, Stf]
+_IntType = (int, np.integer)
 
 
-def is_broadcastable(shape_a, shape_b) -> bool:
+def get_broadcasted_shape(*shapes):
     """
-    Checks if the shapes of two objects are broadcastable.
+    Returns the shape resulting of broadcasting multiple shapes,
+    or None if they're not broadcastable.
 
-    Two shapes are broadcastable if, for each dimension starting from the end,
-    they have the same size value, or if one of them has size 1.
-
-    The function returns True even if some matching "dimensions" are negative.
-    This can be used to compare shapes of Stf/Pwc objects by setting the size
-    of their time_dimension to -1.
+    The shapes are broadcastable if, for each dimension starting from the end,
+    they all have either the same size or a size 1.
 
     Parameters
     ----------
-    shape_a : tuple[int]
-        Shape of one of the objects.
-    shape_b : tuple[int]
-        Shape of the other object.
+    *shapes : tuple[int]
+        Shapes of the objects.
 
     Returns
     -------
-    bool
-        True if shapes of two objects are broadcastable otherwise False
+    tuple[int] or None
+        The resulting broadcasted shape if the shapes are broadcastable, otherwise None.
     """
-    return all(
-        ((dimension_a == dimension_b) or (dimension_a == 1) or (dimension_b == 1))
-        for (dimension_a, dimension_b) in zip(shape_a[::-1], shape_b[::-1])
-    )
+
+    # Obtain largest length of shapes.
+    max_shape_len = max([len(shape) for shape in shapes])
+
+    # Prepend ones to shorter shapes.
+    extended_shapes = [(1,) * (max_shape_len - len(shape)) + shape for shape in shapes]
+
+    # Check that the shapes are broadcastable:
+    # for each position, all shapes hold either the same value or a 1 and another value.
+    broadcasted_shape = []
+    for dimensions in zip(*extended_shapes):
+        dim_set = set(dimensions)
+        if len(dim_set) > 2 or (len(dim_set) == 2 and 1 not in dim_set):
+            return None
+        # The product of each valid set will be either 1 or the non-1 value.
+        broadcasted_shape.append(int(np.prod(list(dim_set))))
+
+    return tuple(broadcasted_shape)
 
 
 def validate_broadcasted_shape(x_shape, y_shape, x_name, y_name):
     """
-    Gets the resulting broadcasted shape for two input shapes.
+    Gets the resulting broadcasted shape for two input shapes,
+    throwing an error if they're not broadcastable.
 
     Parameters
     ----------
@@ -88,31 +107,17 @@ def validate_broadcasted_shape(x_shape, y_shape, x_name, y_name):
         if the two shapes aren't broadcastable.
     """
 
-    if not is_broadcastable(x_shape, y_shape):
+    shape = get_broadcasted_shape(x_shape, y_shape)
+    if shape is None:
         raise QctrlException(
             f"The shapes {x_shape} of {x_name} and {y_shape} of {y_name}"
             " must be broadcastable."
         )
-
-    shape_length = max(len(x_shape), len(y_shape))
-
-    long_x_shape = (1,) * (shape_length - len(x_shape)) + tuple(x_shape)
-    long_y_shape = (1,) * (shape_length - len(y_shape)) + tuple(y_shape)
-
-    shape = (
-        long_x_shape[index] if long_x_shape[index] != 1 else long_y_shape[index]
-        for index in range(shape_length)
-    )
-
-    return tuple(shape)
+    return shape
 
 
 def validate_function_output_shapes(
-    x_batch_shape,
-    x_value_shape,
-    y_batch_shape,
-    y_value_shape,
-    validate_value_shape,
+    x_batch_shape, x_value_shape, y_batch_shape, y_value_shape, validate_value_shape
 ):
     """
     Gets the output batch and value shape for two input shapes of Pwcs/Stfs.
@@ -210,11 +215,11 @@ def validate_tensor_and_function_output_shapes(
 
 def validate_shape(tensor_like, tensor_like_name):
     """
-    Returns the shape of a scalar, np.ndarray, scipy.sparse.coo_matrix, or Tensor node.
+    Returns the shape of a scalar, np.ndarray, scipy.sparse.spmatrix, or Tensor node.
 
     Parameters
     ----------
-    tensor_like : number or np.ndarray or scipy.sparse.coo_matrix or Tensor
+    tensor_like : number or np.ndarray or scipy.sparse.spmatrix or Tensor
         The object whose shape you want to obtain.
     tensor_like_name : str
         The name of the `tensor_like`, used for error message in case the
@@ -344,6 +349,43 @@ def validate_ms_shapes(ion_count, ld_values, ld_name, rd_values, rd_name):
     )
 
 
+def validate_batched_operator(operator, name):
+    """
+    Checks if the input is a valid batched operator.
+
+    Parameters
+    ----------
+    operator : Tensor or np.ndarray
+        An operator.
+    name : str
+        Name of the operator.
+
+    Returns
+    -------
+    tuple[int], tuple[int]
+        The batch shape and value shape of the operator, if all checks are passed.
+
+    Raises
+    ------
+    QctrlException
+        If operator doesn't have numeric element, or is not square.
+    """
+    check_argument_numeric(operator, name)
+    operator_shape = validate_shape(operator, name)
+    check_argument(
+        len(operator_shape) >= 2,
+        f"The {name} must be at least two dimensional.",
+        {name: operator},
+    )
+    check_argument(
+        operator_shape[-1] == operator_shape[-2],
+        f"The {name} must be a square in the last two dimensions.",
+        {name: operator},
+        extras={f"{name} shape": operator_shape},
+    )
+    return operator_shape[:-2], operator_shape[-2:]
+
+
 def check_density_matrix_shape(density_matrix, name):
     """
     Checks the shape of the input density matrix.
@@ -355,19 +397,12 @@ def check_density_matrix_shape(density_matrix, name):
     name : str
         Name of the density matrix.
     """
-    check_argument_numeric(density_matrix, name)
-    density_matrix_shape = validate_shape(density_matrix, name)
+    batch_shape, value_shape = validate_batched_operator(density_matrix, name)
     check_argument(
-        len(density_matrix_shape) in [2, 3],
+        len(batch_shape) in [0, 1],
         f"The {name} must be 2D or 3D with the first axis as the batch dimension.",
         {name: density_matrix},
-        extras={"density matrix shape": density_matrix_shape},
-    )
-    check_argument(
-        density_matrix_shape[-1] == density_matrix_shape[-2],
-        f"The {name} must be a square in the last two dimensions.",
-        {name: density_matrix},
-        extras={"density matrix shape": density_matrix_shape},
+        extras={"density matrix shape": batch_shape + value_shape},
     )
 
 
@@ -388,10 +423,7 @@ def check_oqs_hamiltonian(hamiltonian, initial_density_matrix):
         hamiltonian.value_shape == (system_dimension, system_dimension),
         "The dimension of the Hamiltonian must be compatible with the dimension "
         "of the initial density matrix.",
-        {
-            "hamiltonian": hamiltonian,
-            "initial_density_matrix": initial_density_matrix,
-        },
+        {"hamiltonian": hamiltonian, "initial_density_matrix": initial_density_matrix},
         extras={
             "hamiltonian dimension": hamiltonian.value_shape,
             "initial density matrix dimension": initial_density_matrix.shape,
@@ -403,27 +435,47 @@ def check_lindblad_terms(
     lindblad_terms, hilbert_space_object, hilbert_space_object_name
 ):
     """
-    Check whether Lindblad terms are valid.
+    Check whether Lindblad terms are valid, and whether the number of terms
+    is greater than zero.
 
     Parameters
     ----------
-    lindblad_terms : list[tuple[float, np.ndarray or coo_matrix]]
-        Effective system Hamiltonian.
+    lindblad_terms : list[tuple[float, np.ndarray or spmatrix]]
+        The list of Lindblad terms.
     hilbert_space_object : np.ndarray or Tensor or scipy.sparse.spmatrix
         Object whose last element of its shape is the system's Hilbert space dimension.
     hilbert_space_object_name : str
         The name of the `hilbert_space_object`, used in an error message in case the
         inputs dimensions don't match.
     """
+    check_argument_iterable(lindblad_terms, "lindblad_terms")
+    check_argument(
+        len(lindblad_terms) > 0,
+        "You must provide at least one Lindblad term.",
+        {"lindblad_terms": lindblad_terms},
+    )
 
     system_dimension = hilbert_space_object.shape[-1]
 
-    for index, (rate, operator) in enumerate(lindblad_terms):
+    for index, term in enumerate(lindblad_terms):
         check_argument(
-            rate > 0,
-            ("The decay rate must be positive."),
+            isinstance(term, tuple) and len(term) == 2,
+            "Lindblad terms must be a list of tuples and each tuple must have decay rate and "
+            "operator as the elements.",
             {"lindblad_terms": lindblad_terms},
         )
+        rate, operator = term
+        check_argument(
+            rate > 0,
+            "The decay rate must be positive.",
+            {"lindblad_terms": lindblad_terms},
+        )
+        check_argument(
+            isinstance(operator, (spmatrix, np.ndarray, Tensor)),
+            "Lindblad operator must either be a NumPy array, scipy sparse matrix, or Tensor.",
+            {"lindblad_terms": lindblad_terms},
+        )
+        # Extra check for array.
         check_argument_numeric(operator, f"lindblad_terms[{index}][1]")
         _ = validate_shape(operator, f"lindblad_terms[{index}][1]")
         check_argument_operator(operator, f"lindblad_terms[{index}][1]")
@@ -474,3 +526,240 @@ def validate_inputs_real_fourier_signal(
             "The number of randomized frequencies (if provided) must be greater than zero.",
             {"randomized_frequency_count": randomized_frequency_count},
         )
+
+
+def validate_fock_state_input(dimension, level, offset):
+    """
+    Validate the input for the Fock state node.
+    If all input are valid, this function returns the shape of the Fock state.
+    """
+    from_subsystem = not isinstance(dimension, _IntType)
+
+    if from_subsystem:
+
+        # when from subsystems
+        # dimension must be a list of integers
+        check_argument(
+            _validate_list(dimension, _IntType),
+            "Dimension must either be an integer or a list of integers.",
+            {"dimension": dimension},
+        )
+        _dimension_arr = np.asarray(dimension)
+
+        # offset must be a list of non-negative integers of the size len(dimension)
+        if offset is not None:
+            check_argument(
+                isinstance(offset, list)
+                and _validate_list(offset, _IntType, len(dimension)),
+                "When constructed from subsystems, a non-default offset must be a list of integers "
+                "and have the same size as dimension.",
+                {"dimension": dimension, "offset": offset},
+            )
+
+        _offset_arr = (
+            np.zeros_like(_dimension_arr) if offset is None else np.asarray(offset)
+        )
+        check_argument(
+            np.all(_offset_arr >= 0), "Offset must be non-negative.", {"offset": offset}
+        )
+
+        # level must be List[int] or List[List[int]] or int
+        if isinstance(level, list):
+            check_level_list(level, dimension)
+            _level_arr = np.asarray(level)
+        elif isinstance(level, _IntType):
+            _level_arr = np.repeat(level, len(dimension))
+        else:
+            raise QctrlArgumentsValueError(
+                "Level must be an integer, a list of integers, or a list of lists of integers.",
+                {"level": level},
+            )
+
+        check_argument(
+            np.all(_level_arr >= _offset_arr),
+            "Level must not be smaller than offset.",
+            {"level": level, "offset": offset},
+        )
+        check_argument(
+            np.all(_level_arr < (_dimension_arr + _offset_arr)),
+            "Level must be smaller than dimension+offset.",
+            {"level": level, "dimension": dimension, "offset": offset},
+        )
+
+        return _level_arr.shape[:-1] + (int(np.prod(dimension)),)
+
+    # when direct setup
+    # only integer is allowed for dimension
+    check_argument(
+        isinstance(dimension, _IntType),
+        "Dimension must either be an integer or a list of integers.",
+        {"dimension": dimension},
+    )
+    # offset must be non-negative integers if set
+    _offset = 0 if offset is None else offset
+    check_argument(
+        isinstance(_offset, _IntType),
+        "Dimension and offset must be the same type. They should either be "
+        "integers or a list of integers.",
+        {"dimension": dimension, "offset": offset},
+    )
+    check_argument(_offset >= 0, "Offset must be non-negative.", {"offset": offset})
+
+    # level must be an integer or list of integers that are in [offset, dim+offset)
+    check_argument(
+        isinstance(level, _IntType) or _validate_list(level, _IntType, None),
+        "If the Fock state is not constructed from subsystems, "
+        "level must either be an integer or a list of integers.",
+        {"dimension": dimension, "level": level},
+    )
+    level_arr = np.asarray(level)
+    check_argument(
+        np.all(level_arr >= _offset),
+        "Level must not be smaller than offset.",
+        {"offset": _offset, "level": level},
+    )
+    check_argument(
+        np.all(level_arr < dimension + _offset),
+        "Level must be smaller than dimension + offset.",
+        {"dimension": dimension, "offset": _offset, "level": level},
+    )
+
+    return level_arr.shape + (dimension,)
+
+
+def validate_field_operator_input(parameter, lower_bound, name):
+    """
+    Check the input parameter for creation/annihilation/number operator.
+    """
+    check_argument(
+        isinstance(parameter, _IntType) and parameter >= lower_bound,
+        f"{name} must be an integer and not less than {lower_bound}.",
+        {name: parameter},
+    )
+
+
+def validate_coherent_state_alpha(alpha):
+    """
+    Check whether alpha is valid for generating coherent state.
+    Returns the batch shape if alpha is valid. Otherwise, raise an error.
+    """
+
+    if isinstance(alpha, list):
+        check_argument(
+            _validate_list(alpha, Number),
+            "Alpha must be either a number, a list of them, or a 1D NumPy array.",
+            {"alpha": alpha},
+        )
+        return (len(alpha),)
+    if isinstance(alpha, np.ndarray):
+        check_argument(
+            alpha.ndim == 1,
+            "Alpha must be either a number, a list of them, or a 1D NumPy array.",
+            {"alpha": alpha},
+        )
+        check_argument_numeric(alpha, "alpha")
+        return (len(alpha),)
+    if isinstance(alpha, Number):
+        return ()
+    raise QctrlArgumentsValueError(
+        "Alpha must be either a number, a list of them, or a 1D NumPy array.",
+        {"alpha": alpha},
+    )
+
+
+def check_level_list(level, dimension):
+    """
+    If level is a list, it must be a list of integers and len(level) == len(dimension).
+    If it's a nested list, each element can only be a list of integers of the size len(dimension).
+    """
+    if isinstance(level[0], _IntType):
+        check_argument(
+            _validate_list(level, _IntType, len(dimension)),
+            "Level must be a list of integers with the same length as dimension.",
+            {"level": level, "dimension": dimension},
+        )
+    elif isinstance(level[0], list):
+        for item in level:
+            check_argument(
+                isinstance(item, list)
+                and _validate_list(item, _IntType, len(dimension)),
+                "Each batch of level must be a list of integers with the same "
+                "length as dimension.",
+                {"level": level, "dimension": dimension},
+            )
+    else:
+        raise QctrlArgumentsValueError(
+            "Level must be an integer, a list of integers, or a list of lists of integers.",
+            {"level": level},
+        )
+
+
+def _validate_list(list_, type_, length=None) -> bool:
+    """
+    Check list size and elements all have the same type.
+    """
+    if length is not None and len(list_) != length:
+        return False
+    for item in list_:
+        if not isinstance(item, type_):
+            return False
+    return True
+
+
+def check_operation_axis(axis, shape, tensor, tensor_name):
+    """
+    Certain Tensor operations are applied along the axis of the Tensor.
+    The function checks
+    1. whether the axis is consistent with the shape of Tensor.
+    2. whether there is any repeated item in axis
+    """
+
+    if axis is None:
+        return list(range(len(shape)))
+
+    if isinstance(axis, _IntType):
+        axis = [axis]
+    else:
+        check_argument(
+            isinstance(axis, (list, tuple)),
+            "Non-default axis must be an integer, or a list (tuple) of integers.",
+            {"axis": axis},
+        )
+        for item in axis:
+            check_argument(
+                isinstance(item, _IntType),
+                "Elements of axis must be integers.",
+                {"axis": axis},
+            )
+        axis = list(axis)
+
+    for i, dimension in enumerate(axis):
+        check_argument(
+            -len(shape) <= dimension < len(shape),
+            f"Elements of axis must be valid axes of {tensor_name} (between {-len(shape)} "
+            f"and {len(shape)-1}, inclusive).",
+            {tensor_name: tensor, "axis": axis},
+        )
+        if dimension < 0:
+            axis[i] = dimension + len(shape)
+
+    check_argument(
+        len(set(axis)) == len(axis),
+        f"Elements of axis must refer to unique dimensions of {tensor_name}.",
+        {tensor_name: tensor, "axis": axis},
+    )
+
+    return axis
+
+
+def get_keepdims_operation_shape(shape, axis, keepdims):
+    """
+    Returns the shape of the operations that can keep the dimension of the input tensor.
+    """
+    output_shape = []
+    for i, size in enumerate(shape):
+        if i not in axis:
+            output_shape.append(size)
+        elif keepdims:
+            output_shape.append(1)
+    return tuple(output_shape)
